# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Module for interacting with Terraform files."""


import collections
import json
import os
import subprocess
from typing import Dict, List, Any

from googlecloudsdk.api_lib.scc.remediation_intents import const
from googlecloudsdk.api_lib.scc.remediation_intents import parsers
from googlecloudsdk.command_lib.code import run_subprocess
from googlecloudsdk.command_lib.scc.remediation_intents import errors
from googlecloudsdk.core.util import files


def fetch_tf_files(root_dir: str) -> Dict[str, str]:
  """Fetches all the relevant TF files in the given directory recusively and returns a dictionary of the file paths and contents.

  Args:
    root_dir: The path of the directory to find the TF files in.

  Returns:
    A dictionary of the TF files in the given directory {path: contents}.
  """
  tf_files: Dict[str, str] = {}
  dir_queue = collections.deque([root_dir])

  # Algo: Recursively search the items in the current directory, if it's a
  # directory, add it to the queue. If it is a tf file, read it to the
  # dictionary.
  while dir_queue:
    current_dir = dir_queue.popleft()
    for item in os.listdir(current_dir):
      item_path = os.path.join(current_dir, item)
      if os.path.isdir(item_path):
        if not item.startswith("."):  # Ignore hidden directories
          dir_queue.append(item_path)
      elif os.path.isfile(item_path) and (
          item_path.endswith(".tf") or item_path.endswith(".tfvars")
          and not item_path.startswith(".")
      ):
        tf_files[item_path] = files.ReadFileContents(item_path)
  return tf_files


def fetch_tfstate(dir_path: str)-> json:
  """Fetches the TfState json for the given directory and returns in json format.

  Args:
    dir_path: The path of the directory to fetch the TfState data from.

  Returns:
    The json of the TfState data or throws an exception if there is an error.
  """
  # Fetching tfState data is a two step process:
  # 1. Run terraform init command to initialize the terraform directory.
  # 2. Run terraform show command to fetch the tfstate data.
  try:
    org_dir = os.getcwd()
    os.chdir(dir_path)
    cmd = ["terraform", "init"]  # Step 1
    run_subprocess.GetOutputLines(cmd, timeout_sec=const.TF_CMD_TIMEOUT)
  except Exception as e:
    raise errors.TfStateFetchingError(str(e))
  try:
    cmd = ["terraform", "show", "-json"]  # Step 2
    tfstate_data = run_subprocess.GetOutputLines(
        cmd, timeout_sec=const.TF_CMD_TIMEOUT, strip_output=True
    )
    os.chdir(org_dir)
    return json.loads(tfstate_data[0])
  except Exception as e:
    raise errors.TfStateFetchingError(str(e))


def validate_tf_files(modified_tf_files: Dict[str, str]) -> str:
  """Validates the given TF files and returns the appropriate error message if any.

  Args:
    modified_tf_files: The dictionary of the modified TF files {path: contents}.

  Returns:
    The error message if any in string format, otherwise None.
  """
  # Save the original contents of just the modified TF files.
  original_tf_files: Dict[str, str] = {}
  for file_path, _ in modified_tf_files.items():
    original_tf_files[file_path] = files.ReadFileContents(file_path)

  # Format the modified TF files one by one.
  for file_path, file_content in modified_tf_files.items():
    files.WriteFileContents(file_path, file_content)
    try:
      cmd = ["terraform", "fmt", "-write=true", file_path]
      _ = subprocess.run(
          cmd,
          text=True,
          check=True,
          stdout=subprocess.PIPE,
          stderr=subprocess.PIPE,
      )
    except subprocess.CalledProcessError as e:
      # Restore the original contents of the files in case of error.
      _ = [
          files.WriteFileContents(fp, fc)
          for fp, fc in original_tf_files.items()
      ]

      return const.TF_FMT_ERROR_MSG.format(
          file_path=file_path, stdout=e.stdout, stderr=e.stderr
      )

  # Validate the modified TF files by running terraform validate command.
  cmd = ["terraform", "validate"]
  try:
    _ = subprocess.run(
        cmd,
        text=True,
        check=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
  except subprocess.CalledProcessError as e:
    # Restore the original contents of the files in case of error.
    _ = [
        files.WriteFileContents(fp, fc)
        for fp, fc in original_tf_files.items()
    ]

    return const.TF_VALIDATE_ERROR_MSG.format(
        stdout=e.stdout, stderr=e.stderr
    )

  # Restore the original contents of the files finally.
  _ = [files.WriteFileContents(fp, fc) for fp, fc in original_tf_files.items()]
  return None


def get_resources_from_tfstate(
    tfstate_json: Dict[str, Any],
) -> List[Dict[str, Any]]:
  """Traverses the TfState json and returns a list of resources in json format.

  Args:
    tfstate_json: The json of the TfState data. Structure:
                  {
                      "values": {
                          "root_module": {
                              "resources": [ ... ],  # List of resources
                              "child_modules": [     # List of nested modules
                                  {
                                      "resources": [ ... ],
                                      "child_modules": [ ... ]
                                  }
                              ]
                          }
                      }
                  }
  Returns:
    A list of json objects, each representing a resource in the TfState.
    or an empty list if there are no resources in the TfState or if the TfState
    is not in the expected format.
  """

  all_resources = []
  # Recursive function to traverse the tfstate data and extract the resources.
  def traverse(module: Dict[str, Any]):
    if "resources" in module:
      all_resources.extend(module["resources"])
    if "child_modules" in module:
      for child in module["child_modules"]:
        traverse(child)

  root_module = tfstate_json.get("values", {}).get("root_module", {})
  traverse(root_module)
  return all_resources


def parse_tf_file(dir_path: str, finding_data) -> str:
  """Parses the tfstate file for the given finding.

  Args:
    dir_path: The path of the directory to parse the tfstate file from.
    finding_data: SCC Finding data in form of class
      (securityposture.messages.Finding).

  Returns:
    The structured data depending on the finding category, in string format. If
    the finding category is not supported, returns an empty string.
  """
  tftstate_json = fetch_tfstate(dir_path)
  resources = get_resources_from_tfstate(tftstate_json)

  # Mapping of finding category to the parser function.
  parser_map = {  # category: parser_function
      **{
          category: parsers.iam_recommender_parser
          for category in const.IAM_RECOMMENDER_FINDINGS
      },
      **{
          category: parsers.firewall_parser
          for category in const.FIREWALL_FINDINGS
      },
  }
  # Each parser function takes the list of resources and the finding data as
  # input and returns the structured data in string format.
  if finding_data.category in parser_map:
    return parser_map[finding_data.category](resources, finding_data)
  return ""



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for reading and manipulating Assured Workloads data controls."""


from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.base import ReleaseTrack


@base.ReleaseTracks(ReleaseTrack.GA, ReleaseTrack.BETA, ReleaseTrack.ALPHA)
class Assured(base.Group):
  """Read and manipulate Assured Workloads data controls."""

  category = base.SECURITY_CATEGORY

  detailed_help = {
      'DESCRIPTION':
          """
        Read and manipulate Assured Workloads data controls.
    """,
  }

  def Filter(self, context, args):
    # TODO(b/190525911):  Determine if command group works with project number
    base.RequireProjectID(args)
    del context, args



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dataform resource handler."""

from typing import Any, Dict

from apitools.base.protorpclite import messages
from apitools.base.py import encoding
from googlecloudsdk.command_lib.orchestration_pipelines.handlers import base
from googlecloudsdk.core import log


class DataformRepositoryHandler(base.GcpResourceHandler):
  """Handler for Dataform Repository resources."""

  _documentation_uri = (
      "https://cloud.google.com/dataform/reference/rest/"
      "v1beta1/projects.locations.repositories"
  )
  description = "Dataform Repository resources."

  def build_get_request(self) -> messages.Message:
    return self.messages.DataformProjectsLocationsRepositoriesGetRequest(
        name=self._get_resource_name()
    )

  def build_create_request(
      self, resource_message: messages.Message
  ) -> messages.Message:
    return self.messages.DataformProjectsLocationsRepositoriesCreateRequest(
        parent=self._get_parent_path(),
        repository=resource_message,
        repositoryId=self.get_resource_id(),
    )

  def build_update_request(
      self,
      existing_resource: messages.Message,
      resource_message: messages.Message,
      changed_fields: list[str],
  ) -> messages.Message:
    resource_message.name = existing_resource.name
    return self.messages.DataformProjectsLocationsRepositoriesPatchRequest(
        name=existing_resource.name,
        repository=resource_message,
        updateMask=",".join(changed_fields),
    )


class DataformReleaseConfigHandler(base.GcpResourceHandler):
  """Handler for Dataform ReleaseConfig resources."""

  _documentation_uri = (
      "https://cloud.google.com/dataform/reference/rest/"
      "v1beta1/projects.locations.repositories.releaseConfigs"
  )
  description = (
      "Dataform ReleaseConfig resources.\n"
      "Special handling:\n"
      " - releaseCompilationResult: setting to 'auto' in definition triggers "
      "compilation after resource creation or update"
  )

  def build_get_request(self) -> messages.Message:
    return self.messages.DataformProjectsLocationsRepositoriesReleaseConfigsGetRequest(
        name=self._get_resource_name()
    )

  def build_create_request(
      self, resource_message: messages.Message
  ) -> messages.Message:
    return self.messages.DataformProjectsLocationsRepositoriesReleaseConfigsCreateRequest(
        parent=self._get_parent_path(),
        releaseConfig=resource_message,
        releaseConfigId=self.get_resource_id(),
    )

  def build_update_request(
      self,
      existing_resource: messages.Message,
      resource_message: messages.Message,
      changed_fields: list[str],
  ) -> messages.Message:
    resource_message.name = existing_resource.name
    return self.messages.DataformProjectsLocationsRepositoriesReleaseConfigsPatchRequest(
        name=existing_resource.name,
        releaseConfig=resource_message,
        updateMask=",".join(changed_fields),
    )

  def get_local_definition(self) -> Dict[str, Any]:
    definition = super().get_local_definition()
    if "releaseCompilationResult" in definition:
      self._release_compilation_result = definition.pop(
          "releaseCompilationResult"
      )
    return definition

  def compare(
      self, existing_resource: Any, local_definition: dict[str, Any]
  ) -> list[str]:
    diffs = super().compare(existing_resource, local_definition)
    if diffs:
      return diffs

    if getattr(self, "_release_compilation_result", None) == "auto":
      existing_dict = existing_resource
      if not isinstance(existing_resource, dict):
        existing_dict = encoding.MessageToDict(existing_resource)

      compilation_result_name = existing_dict.get("releaseCompilationResult")
      if not compilation_result_name:
        log.status.Print(
            "     [+] Server has no compilation result. Forcing compilation..."
        )
        return ["releaseCompilationResult"]

      # Fetch compilation result to verify commit
      try:
        get_request_type = (
            self.messages.DataformProjectsLocationsRepositoriesCompilationResultsGetRequest
        )
        get_request = get_request_type(name=compilation_result_name)
        compilation_result = (
            self.client.projects_locations_repositories_compilationResults.Get(
                get_request
            )
        )

        if compilation_result.compilationErrors:
          log.status.Print(
              f"     [+] Found {len(compilation_result.compilationErrors)} "
              "compilation errors in existing results. Forcing recompilation..."
          )
          return ["releaseCompilationResult"]

        expected_commit = local_definition.get("gitCommitish")
        server_commit = compilation_result.resolvedGitCommitSha

        # Clever compare (match short vs full)
        if server_commit and expected_commit:
          if (not server_commit.startswith(expected_commit) and
              not expected_commit.startswith(server_commit)):
            log.status.Print(
                f"     [+] Compilation commit {server_commit} != expected "
                f"{expected_commit}. Forcing recompilation..."
            )
            return ["releaseCompilationResult"]
      except Exception as e:  # pylint: disable=broad-except
        log.status.Print(
            f"     [!] Failed to verify compilation result: {e}. "
            "Forcing recompilation..."
        )
        return ["releaseCompilationResult"]

    return []

  def post_deploy(self, api_response: Any, created: bool) -> None:
    if getattr(self, "_release_compilation_result", None) == "auto":
      log.status.Print(
          f"     [+] Triggering compilation for release config "
          f"'{self.resource.name}'..."
      )
      request = self.messages.DataformProjectsLocationsRepositoriesCompilationResultsCreateRequest(
          parent=self._get_parent_path(),
          compilationResult=self.messages.CompilationResult(
              releaseConfig=self._get_resource_name()
          ),
      )
      compilation_result = (
          self.client.projects_locations_repositories_compilationResults.Create(
              request=request
          )
      )
      if compilation_result.compilationErrors:
        error_details = "\n".join([
            f"  - {err.path or 'Unknown file'}: {err.message}"
            for err in compilation_result.compilationErrors
        ])
        raise ValueError(
            "Dataform compilation failed with following errors:\n"
            f"{error_details}\nStopping deployment."
        )
      local_def = self.get_local_definition()
      git_commitish = local_def.get("gitCommitish")
      patch_request = self.messages.DataformProjectsLocationsRepositoriesReleaseConfigsPatchRequest(
          name=self._get_resource_name(),
          releaseConfig=self.messages.ReleaseConfig(
              releaseCompilationResult=compilation_result.name,
              gitCommitish=git_commitish
          ),
          updateMask="releaseCompilationResult",
      )
      self.client.projects_locations_repositories_releaseConfigs.Patch(
          request=patch_request
      )


class DataformWorkflowConfigHandler(base.GcpResourceHandler):
  """Handler for Dataform WorkflowConfig resources."""

  _documentation_uri = (
      "https://cloud.google.com/dataform/reference/rest/"
      "v1beta1/projects.locations.repositories.workflowConfigs"
  )
  description = (
      "Dataform WorkflowConfig resources.\n"
      "Definition handling:\n"
      " - releaseConfig: accepts short ID of config within the same "
      "repository instead of full URI"
  )

  def build_get_request(self) -> messages.Message:
    return self.messages.DataformProjectsLocationsRepositoriesWorkflowConfigsGetRequest(
        name=self._get_resource_name()
    )

  def get_local_definition(self) -> Dict[str, Any]:
    definition = super().get_local_definition()
    if ("releaseConfig" in definition and
        "/" not in definition["releaseConfig"]):
      release_config_id = definition["releaseConfig"]
      definition["releaseConfig"] = (
          f"{self._get_parent_path()}/releaseConfigs/{release_config_id}"
      )
    return definition

  def build_create_request(
      self, resource_message: messages.Message
  ) -> messages.Message:
    return self.messages.DataformProjectsLocationsRepositoriesWorkflowConfigsCreateRequest(
        parent=self._get_parent_path(),
        workflowConfig=resource_message,
        workflowConfigId=self.get_resource_id(),
    )

  def build_update_request(
      self,
      existing_resource: messages.Message,
      resource_message: messages.Message,
      changed_fields: list[str],
  ) -> messages.Message:
    resource_message.name = existing_resource.name
    return self.messages.DataformProjectsLocationsRepositoriesWorkflowConfigsPatchRequest(
        name=existing_resource.name,
        workflowConfig=resource_message,
        updateMask=",".join(changed_fields),
    )


class DataformWorkspaceHandler(base.GcpResourceHandler):
  """Handler for Dataform Workspace resources."""

  _documentation_uri = (
      "https://cloud.google.com/dataform/reference/rest/"
      "v1beta1/projects.locations.repositories.workspaces"
  )
  description = "Dataform Workspace resources."

  def build_get_request(self) -> messages.Message:
    req_cls = (
        self.messages.DataformProjectsLocationsRepositoriesWorkspacesGetRequest
    )
    return req_cls(name=self._get_resource_name())

  def build_create_request(
      self, resource_message: messages.Message
  ) -> messages.Message:
    return self.messages.DataformProjectsLocationsRepositoriesWorkspacesCreateRequest(
        parent=self._get_parent_path(),
        workspace=resource_message,
        workspaceId=self.get_resource_id(),
    )

  def build_update_request(
      self,
      existing_resource: messages.Message,
      resource_message: messages.Message,
      changed_fields: list[str],
  ) -> messages.Message:
    # Dataform Workspaces only support Create/Get/Delete according to API,
    # so we should error on update. Note: dataform v1beta1 might have different
    # methods but we fall back to generic approach. If Patch is unsupported
    # it will raise exception during execution.
    resource_message.name = existing_resource.name
    return self.messages.DataformProjectsLocationsRepositoriesWorkspacesPatchRequest(
        name=existing_resource.name,
        workspace=resource_message,
        updateMask=",".join(changed_fields),
    )



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The super-group for the compute CLI."""


from googlecloudsdk.api_lib.compute import transforms
from googlecloudsdk.calliope import base


DETAILED_HELP = {
    'DESCRIPTION': """
        The gcloud compute command group lets you create, configure, and
        manipulate Compute Engine virtual machine (VM) instances.

        With Compute Engine, you can create and run VMs
        on Google's infrastructure. Compute Engine offers scale, performance,
        and value that lets you launch large compute clusters on
        Google's infrastructure.

        For more information about Compute Engine, see the
        [Compute Engine overview](https://cloud.google.com/compute/)
        and the
        [Compute Engine user documentation](https://cloud.google.com/compute/docs/).
        """,
}


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.GA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.PREVIEW,
)
class Compute(base.Group):
  """Create and manipulate Compute Engine resources."""
  detailed_help = DETAILED_HELP

  category = base.COMPUTE_CATEGORY

  @staticmethod
  def Args(parser):
    parser.display_info.AddTransforms(transforms.GetTransforms())

  def Filter(self, context, args):
    # TODO(b/190528962):  Determine if command group works with project number
    base.RequireProjectID(args)
    del context, args
    base.DisableUserProjectQuota()

    self.EnableSelfSignedJwtForTracks(
        [base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA]
    )



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The Browse command."""



from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.app import browser_dispatcher
from googlecloudsdk.command_lib.app import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


class Browse(base.Command):
  """Open the specified versions in a browser."""

  detailed_help = {
      'EXAMPLES': """\
          To show version `v1` for the default service in the browser, run:

              $ {command} v1

          To show version `v1` of a specific service in the browser, run:

              $ {command} v1 --service="myService"

          To show multiple versions side-by-side, run:

              $ {command} v1 v2 --service="myService"
          """,
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""
          table(
            version:label=VERSION,
            url:label=URL
          )
    """)
    flags.LAUNCH_BROWSER.AddToParser(parser)
    parser.add_argument(
        'versions',
        nargs='+',
        help="""\
        The versions to open (optionally filtered by the --service flag).""")
    parser.add_argument('--service', '-s',
                        help=('If specified, only open versions from the '
                              'given service. If not specified, use the '
                              'default service.'))

  def Run(self, args):
    """Launch a browser, or return a table of URLs to print."""
    project = properties.VALUES.core.project.Get(required=True)
    outputs = []
    for version in args.versions:
      outputs.append(browser_dispatcher.BrowseApp(project,
                                                  args.service,
                                                  version,
                                                  args.launch_browser))
    if any(outputs):
      if args.launch_browser:
        # We couldn't find a browser to launch
        log.status.Print(
            'Did not detect your browser. Go to these links to view your app:')
      return outputs
    return None



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to get a serverless ray job in Vertex AI."""


from googlecloudsdk.api_lib.ai.serverless_ray_jobs import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai.serverless_ray_jobs import flags


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.UniverseCompatible
class DescribeGA(base.DescribeCommand):
  """Get detailed information about the servless ray job by given id.

  ## EXAMPLES

  To get a job ``123'' under project ``example'' in region
  ``us-central1'', run:

    $ {command} 123 --project=example --region=us-central1
  """

  _api_version = constants.GA_VERSION

  @staticmethod
  def Args(parser):
    flags.AddServerlessRayJobResourceArg(parser, 'to describe')

  def Run(self, args):
    serverless_ray_job_ref = args.CONCEPTS.serverless_ray_job.Parse()
    # TODO(b/390723702): Add request validation
    region = serverless_ray_job_ref.AsDict()['locationsId']

    with endpoint_util.AiplatformEndpointOverrides(
        version=self._api_version, region=region):
      response = client.ServerlessRayJobsClient(version=self._api_version).Get(
          serverless_ray_job_ref.RelativeName()
      )
      return response


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class DescribePreGA(DescribeGA):
  """Get detailed information about the custom job by given id.

  ## EXAMPLES

  To get a job ``123'' under project ``example'' in region
  ``us-central1'', run:

    $ {command} 123 --project=example --region=us-central1
  """

  _api_version = constants.BETA_VERSION



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating wire groups."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.interconnects.wire_groups import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.interconnects.cross_site_networks import flags as cross_site_network_flags
from googlecloudsdk.command_lib.compute.interconnects.wire_groups import flags
from googlecloudsdk.core import properties


_DETAILED_HELP = {
    'DESCRIPTION': """\
        *{command}* is used to create wire groups. A wire group represents a
        group of redundant wires between interconnects in two different metros.
        Each WireGroup belongs to a CrossSiteNetwork.

        For an example, refer to the *EXAMPLES* section below.
        """,
    'EXAMPLES': """\
        To create a wire group, run:

          $ {command} example-wg \
              --cross-site-network=example-csn
              --bandwidth-unmetered=1
        """,
}


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
class Create(base.CreateCommand):
  """Create a Compute Engine wire group.

  *{command}* is used to create wire groups. A wire group represents a
  group of redundant wires between interconnects in two different metros.
  Each WireGroup belongs to a CrossSiteNetwork.
  """

  # Framework override.
  detailed_help = _DETAILED_HELP

  WIRE_GROUP_ARG = None
  CROSS_SITE_NETWORK_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.CROSS_SITE_NETWORK_ARG = (
        cross_site_network_flags.CrossSiteNetworkArgumentForOtherResource()
    )
    cls.CROSS_SITE_NETWORK_ARG.AddArgument(parser)
    cls.WIRE_GROUP_ARG = flags.WireGroupArgument(plural=False)
    cls.WIRE_GROUP_ARG.AddArgument(parser, operation_type='create')

    flags.AddDescription(parser)
    flags.AddBandwidthUnmetered(parser)
    flags.AddBandwidthAllocation(parser)
    flags.AddFaultResponse(parser)
    flags.AddAdminEnabled(parser)
    flags.AddValidateOnly(parser)

  def Collection(self):
    return 'compute.wireGroups'

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.WIRE_GROUP_ARG.ResolveAsResource(
        args,
        holder.resources,
        default_scope=compute_scope.ScopeEnum.GLOBAL,
        additional_params={'crossSiteNetwork': args.cross_site_network},
    )

    project = properties.VALUES.core.project.GetOrFail()
    wire_group = client.WireGroup(
        ref, project, args.cross_site_network, compute_client=holder.client
    )

    return wire_group.Create(
        description=args.description,
        # Need to rename type as it conflicts with python built in type()
        wire_group_type=getattr(args, 'type', None),
        bandwidth_unmetered=args.bandwidth_unmetered,
        bandwidth_metered=getattr(args, 'bandwidth_metered', None),
        fault_response=args.fault_response,
        admin_enabled=args.admin_enabled,
        network_service_class=getattr(args, 'network_service_class', None),
        bandwidth_allocation=getattr(args, 'bandwidth_allocation', None),
        validate_only=args.validate_only,
    )


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateBeta(Create):
  """Create a Compute Engine wire group."""

  @classmethod
  def Args(cls, parser):
    super().Args(parser)
    flags.AddType(parser)


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(CreateBeta):
  """Create a Compute Engine wire group.

  *{command}* is used to create wire groups. A wire group represents a
  group of redundant wires between interconnects in two different metros.
  Each WireGroup belongs to a CrossSiteNetwork.
  """

  @classmethod
  def Args(cls, parser):
    super().Args(parser)
    flags.AddBandwidthMetered(parser)
    flags.AddNetworkServiceClass(parser)



"""Generated message classes for composerflex version v1alpha1.

Manages workflows on Google Cloud Platform using Apache Airflow. Composer Flex
additionally provides per-workflow security, auto-scaling, and native GCP
tooling for workflows and tasks.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'composerflex'


class AirflowDag(_messages.Message):
  r"""A specification of an Apache Airflow DAG.

  Fields:
    data: The contents of a Python module or ZIP archive that specifies an
      Airflow DAG.
  """

  data = _messages.BytesField(1)


class ComposerflexProjectsLocationsContextsCreateRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsContextsCreateRequest object.

  Fields:
    context: A Context resource to be passed as the request body.
    parent: Parent resource of the context to create. The parent must be of
      the form "projects/{projectId}/locations/{locationId}".
  """

  context = _messages.MessageField('Context', 1)
  parent = _messages.StringField(2, required=True)


class ComposerflexProjectsLocationsContextsDeleteRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsContextsDeleteRequest object.

  Fields:
    name: The resource name of the context to delete. Must be in the form
      "projects/{projectId}/locations/{locationId}/contexts/{contextId}."
  """

  name = _messages.StringField(1, required=True)


class ComposerflexProjectsLocationsContextsGetRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsContextsGetRequest object.

  Fields:
    name: The resource name of the context to retrieve. Must be in the form
      "projects/{projectId}/locations/{locationId}/contexts/{contextId}."
  """

  name = _messages.StringField(1, required=True)


class ComposerflexProjectsLocationsContextsListRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsContextsListRequest object.

  Fields:
    filter: Filter will remain internal until its future implementation.
    orderBy: Optional. Specifies the ordering of results following syntax at
      https://cloud.google.com/apis/design/design_patterns#sorting_order.
      Order by will remain internal until its future implementation.
    pageSize: The maximum number of contexts to return.
    pageToken: Optional. The next_page_token returned from a previous List
      request.
    parent: List contexts in the given parent resource. Parent must be in the
      form "projects/{projectId}/locations/{locationId}".
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ComposerflexProjectsLocationsWorkflowsCreateRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsWorkflowsCreateRequest object.

  Fields:
    parent: Parent resource of the workflow to create. The parent must be of
      the form "projects/{projectId}/locations/{locationId}".
    workflow: A Workflow resource to be passed as the request body.
  """

  parent = _messages.StringField(1, required=True)
  workflow = _messages.MessageField('Workflow', 2)


class ComposerflexProjectsLocationsWorkflowsDeleteRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsWorkflowsDeleteRequest object.

  Fields:
    name: The resource name of the workflow to delete. Must be in the form
      "projects/{projectId}/locations/{locationId}/workflows/{workflowId}."
  """

  name = _messages.StringField(1, required=True)


class ComposerflexProjectsLocationsWorkflowsGetRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsWorkflowsGetRequest object.

  Fields:
    name: The resource name of the workflow to retrieve. Must be in the form
      "projects/{projectId}/locations/{locationId}/workflows/{workflowId}."
  """

  name = _messages.StringField(1, required=True)


class ComposerflexProjectsLocationsWorkflowsListRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsWorkflowsListRequest object.

  Fields:
    filter: Filter will remain internal until its future implementation.
    orderBy: Optional. Specifies the ordering of results following syntax at
      https://cloud.google.com/apis/design/design_patterns#sorting_order.
      Order by will remain internal until its future implementation.
    pageSize: The maximum number of workflows to return.
    pageToken: Optional. The next_page_token returned from a previous List
      request.
    parent: List workflows in the given parent resource. Parent must be in the
      form "projects/{projectId}/locations/{locationId}".
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class ComposerflexProjectsLocationsWorkflowsRunsGetRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsWorkflowsRunsGetRequest object.

  Fields:
    name: The resource name for the WorkflowRun in the form: "projects/{projec
      tId}/locations/{locationId}/workflows/{workflowId}/runs/{runId}".
  """

  name = _messages.StringField(1, required=True)


class ComposerflexProjectsLocationsWorkflowsRunsListRequest(_messages.Message):
  r"""A ComposerflexProjectsLocationsWorkflowsRunsListRequest object.

  Fields:
    filter: Filter will remain internal until its future implementation.
    orderBy: Optional. Specifies the ordering of results following syntax at
      https://cloud.google.com/apis/design/design_patterns#sorting_order.
      Order by will remain internal until its future implementation.
    pageSize: The maximum number of workflows to return.
    pageToken: Optional. The next_page_token returned from a previous List
      request.
    parent: The parent selector for the WorkflowRun. Supports parent in the
      form:
      "projects/{projectId}/locations/{locationId}/workflows/{workflowId}".
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class Connection(_messages.Message):
  r"""Represents an Airflow Connection. Usage of the fields within this
  message is described in Airflow documentation:
  https://airflow.apache.org/howto/connection/index.html#editing-a-connection-
  with-the-ui

  Enums:
    TypeValueValuesEnum: The type of connection specified. This refers to the
      service for which this connection is created.

  Fields:
    extra: The Airflow extra field for the connection.
    host: The target host for the service.
    id: The identifier for the connection.
    login: The username if the service requires authentication.
    password: Input only. Users can provide the password in plain-text. It
      will be used to set the password in Airflow, but will not be stored for
      retrieval on Google systems.
    port: The port for the service.
    schema: The Airflow schema field for the connection.
    type: The type of connection specified. This refers to the service for
      which this connection is created.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""The type of connection specified. This refers to the service for which
    this connection is created.

    Values:
      TYPE_UNSPECIFIED: Unspecified connection type.
      DOCKER: Docker connection type.
      FILE: File connection type.
      FTP: FTP connection type.
      GOOGLE_CLOUD_PLATFORM: GCP connection type.
      HDFS: HDFS connection type.
      HTTP: HTTP connection type.
      HIVE_CLI: Hive CLI connection type.
      HIVE_METASTORE: Hive Metastore connection type.
      HIVE_SERVER_2: Hive server connection type.
      JDBC: JDBC connection type.
      JENKINS: Jenkins connection type.
      MYSQL: MySQL connection type.
      POSTGRES: Postgres connection type.
      ORACLE: Oracle connection type.
      VERTICA: Vertica connection type.
      PRESTO: Presto connection type.
      S3: Amazon S3 connection type.
      SAMBA: Samba connection type.
      SQLITE: SQLite connection type.
      SSH: SSH connection type.
      CLOUDANT: Cloudant connection type.
      MSSQL: MsSQL connection type.
      MESOS_FRAMEWORK_ID: Mesos Framework connection type.
      JIRA: Jira connection type.
      REDIS: Redis connection type.
      WASB: Wasb connection type.
      DATABRICKS: Databricks connection type.
      AWS: AWS connection type.
      EMR: EMR connection type.
      SNOWFLAKE: Snowflake connection type.
      SEGMENT: Segment connection type.
      AZURE_DATA_LAKE: Azure Data Lake connection type.
      CASSANDRA: Cassandra connection type.
    """
    TYPE_UNSPECIFIED = 0
    DOCKER = 1
    FILE = 2
    FTP = 3
    GOOGLE_CLOUD_PLATFORM = 4
    HDFS = 5
    HTTP = 6
    HIVE_CLI = 7
    HIVE_METASTORE = 8
    HIVE_SERVER_2 = 9
    JDBC = 10
    JENKINS = 11
    MYSQL = 12
    POSTGRES = 13
    ORACLE = 14
    VERTICA = 15
    PRESTO = 16
    S3 = 17
    SAMBA = 18
    SQLITE = 19
    SSH = 20
    CLOUDANT = 21
    MSSQL = 22
    MESOS_FRAMEWORK_ID = 23
    JIRA = 24
    REDIS = 25
    WASB = 26
    DATABRICKS = 27
    AWS = 28
    EMR = 29
    SNOWFLAKE = 30
    SEGMENT = 31
    AZURE_DATA_LAKE = 32
    CASSANDRA = 33

  extra = _messages.StringField(1)
  host = _messages.StringField(2)
  id = _messages.StringField(3)
  login = _messages.StringField(4)
  password = _messages.StringField(5)
  port = _messages.IntegerField(6, variant=_messages.Variant.INT32)
  schema = _messages.StringField(7)
  type = _messages.EnumField('TypeValueValuesEnum', 8)


class Context(_messages.Message):
  r"""A resource to hold configurations that are common across multiple
  workflows.

  Messages:
    AirflowConfigOverridesValue: Optional. A user defined mapping of key value
      pairs for Airflow Configurations.
    AirflowVariablesValue: Optional. Key value pairs to make available to a
      workflow. These are an application-level concept and are different from
      environment variables, which are a process-level concept.
    ConnectionsValue: Optional. A mapping from connection ID to Connections to
      make available to the workflow. Each map entry's key should match the
      `id` field of the // value.
    EnvVariablesValue: Optional. Environment variables to provide to the
      processes executing the workflow. Environment variable names must match
      the regular expression "a-zA-Z_*". They cannot specify Apache Airflow
      configuration overrides (they cannot match the regular expression
      `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`)
    LabelsValue: Optional. User-defined labels for this context. There can be
      no more than 64 labels entries. Entries of the labels map are UTF8
      strings that comply with the following restrictions: * Keys must conform
      to regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
      [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
      constrained to be <= 128 bytes in size.
    PoolsValue: Optional. A mapping from pool ID to Pools to make available to
      the workflow. Each map entry's key should match the `id` field of the
      value.

  Fields:
    airflowConfigOverrides: Optional. A user defined mapping of key value
      pairs for Airflow Configurations.
    airflowVariables: Optional. Key value pairs to make available to a
      workflow. These are an application-level concept and are different from
      environment variables, which are a process-level concept.
    connections: Optional. A mapping from connection ID to Connections to make
      available to the workflow. Each map entry's key should match the `id`
      field of the // value.
    createTime: Output only. The time the context was created.
    envVariables: Optional. Environment variables to provide to the processes
      executing the workflow. Environment variable names must match the
      regular expression "a-zA-Z_*". They cannot specify Apache Airflow
      configuration overrides (they cannot match the regular expression
      `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`)
    labels: Optional. User-defined labels for this context. There can be no
      more than 64 labels entries. Entries of the labels map are UTF8 strings
      that comply with the following restrictions: * Keys must conform to
      regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
      [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
      constrained to be <= 128 bytes in size.
    name: The resource name of the context, in the form:
      "projects/{projectId}/locations/{locationId}/contexts/{contextId}"
    pools: Optional. A mapping from pool ID to Pools to make available to the
      workflow. Each map entry's key should match the `id` field of the value.
    updateTime: Output only. The time the context was last updated.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AirflowConfigOverridesValue(_messages.Message):
    r"""Optional. A user defined mapping of key value pairs for Airflow
    Configurations.

    Messages:
      AdditionalProperty: An additional property for a
        AirflowConfigOverridesValue object.

    Fields:
      additionalProperties: Additional properties of type
        AirflowConfigOverridesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AirflowConfigOverridesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AirflowVariablesValue(_messages.Message):
    r"""Optional. Key value pairs to make available to a workflow. These are
    an application-level concept and are different from environment variables,
    which are a process-level concept.

    Messages:
      AdditionalProperty: An additional property for a AirflowVariablesValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        AirflowVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AirflowVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ConnectionsValue(_messages.Message):
    r"""Optional. A mapping from connection ID to Connections to make
    available to the workflow. Each map entry's key should match the `id`
    field of the // value.

    Messages:
      AdditionalProperty: An additional property for a ConnectionsValue
        object.

    Fields:
      additionalProperties: Additional properties of type ConnectionsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ConnectionsValue object.

      Fields:
        key: Name of the additional property.
        value: A Connection attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('Connection', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class EnvVariablesValue(_messages.Message):
    r"""Optional. Environment variables to provide to the processes executing
    the workflow. Environment variable names must match the regular expression
    "a-zA-Z_*". They cannot specify Apache Airflow configuration overrides
    (they cannot match the regular expression
    `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`)

    Messages:
      AdditionalProperty: An additional property for a EnvVariablesValue
        object.

    Fields:
      additionalProperties: Additional properties of type EnvVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a EnvVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels for this context. There can be no more
    than 64 labels entries. Entries of the labels map are UTF8 strings that
    comply with the following restrictions: * Keys must conform to regexp:
    \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
    [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
    constrained to be <= 128 bytes in size.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class PoolsValue(_messages.Message):
    r"""Optional. A mapping from pool ID to Pools to make available to the
    workflow. Each map entry's key should match the `id` field of the value.

    Messages:
      AdditionalProperty: An additional property for a PoolsValue object.

    Fields:
      additionalProperties: Additional properties of type PoolsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a PoolsValue object.

      Fields:
        key: Name of the additional property.
        value: A Pool attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('Pool', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  airflowConfigOverrides = _messages.MessageField('AirflowConfigOverridesValue', 1)
  airflowVariables = _messages.MessageField('AirflowVariablesValue', 2)
  connections = _messages.MessageField('ConnectionsValue', 3)
  createTime = _messages.StringField(4)
  envVariables = _messages.MessageField('EnvVariablesValue', 5)
  labels = _messages.MessageField('LabelsValue', 6)
  name = _messages.StringField(7)
  pools = _messages.MessageField('PoolsValue', 8)
  updateTime = _messages.StringField(9)


class ListContextsResponse(_messages.Message):
  r"""Response for a ListContextsRequest.

  Fields:
    contexts: The list of contexts returned.
    nextPageToken: The page token used to query for the next page if one
      exists.
  """

  contexts = _messages.MessageField('Context', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListWorkflowRunsResponse(_messages.Message):
  r"""Response for ListWorkflowRunsRequest.

  Fields:
    nextPageToken: The page token used to query for the next page if one
      exists.
    workflowRuns: The returned list of WorkflowRuns.
  """

  nextPageToken = _messages.StringField(1)
  workflowRuns = _messages.MessageField('WorkflowRun', 2, repeated=True)


class ListWorkflowsResponse(_messages.Message):
  r"""Response to ListWorkflowsRequest.

  Fields:
    nextPageToken: The page token used to query for the next page if one
      exists.
    workflows: The list of workflows returned.
  """

  nextPageToken = _messages.StringField(1)
  workflows = _messages.MessageField('Workflow', 2, repeated=True)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class Pool(_messages.Message):
  r"""Represents an Airflow Pool.

  Fields:
    id: The identifier for the pool.
    slots: The number of slots available for the pool.
  """

  id = _messages.StringField(1)
  slots = _messages.IntegerField(2, variant=_messages.Variant.INT32)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class Workflow(_messages.Message):
  r"""A Composer workflow resource.

  Enums:
    StateValueValuesEnum: Output only. The current state of the workflow.

  Messages:
    LabelsValue: Optional. User-defined labels for this workflow. Workflow
      runs instantiated from this workflow will inherit these labels. The
      labels map can contain no more than 64 entries. Entries of the labels
      map are UTF8 strings that comply with the following restrictions: * Keys
      must conform to regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to
      regexp: [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are
      additionally constrained to be <= 128 bytes in size.

  Fields:
    airflowDag: A representation of the workflow as Python source code
      defining an Airflow DAG.
    airflowVersion: The version of Airflow at which to run the DAG. For
      example, "1.10.0".
    composerVersion: Output only. The version of Composer that created this
      workflow.
    context: The context to use when executing this workflow. The context
      resides in the same project and location as this workflow. The context
      must be specified as the full resource name. The format is:
      projects/{projectId}/locations/{locationId}/contexts/{contextId}
    createTime: Output only. The time at which this workflow was created.
    labels: Optional. User-defined labels for this workflow. Workflow runs
      instantiated from this workflow will inherit these labels. The labels
      map can contain no more than 64 entries. Entries of the labels map are
      UTF8 strings that comply with the following restrictions: * Keys must
      conform to regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
      [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
      constrained to be <= 128 bytes in size.
    name: The resource name of the workflow, in the form:
      "projects/{projectId}/locations/{locationId}/workflows/{workflowId}"
    pythonVersion: The major version of Python used to run Apache Airflow
      processes. Can be set to '2' or '3'. If not specified, the default is
      '2'.
    state: Output only. The current state of the workflow.
    updateTime: Output only. The time at which this workflow was last updated.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current state of the workflow.

    Values:
      STATE_UNSPECIFIED: The state of the workflow is unknown.
      CREATING: The workflow is being created and cannot execute.
      ACTIVE: The workflow is available for execution.
      ERROR: The workflow is in an error state and cannot execute.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    ACTIVE = 2
    ERROR = 3

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels for this workflow. Workflow runs
    instantiated from this workflow will inherit these labels. The labels map
    can contain no more than 64 entries. Entries of the labels map are UTF8
    strings that comply with the following restrictions: * Keys must conform
    to regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
    [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
    constrained to be <= 128 bytes in size.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  airflowDag = _messages.MessageField('AirflowDag', 1)
  airflowVersion = _messages.StringField(2)
  composerVersion = _messages.StringField(3)
  context = _messages.StringField(4)
  createTime = _messages.StringField(5)
  labels = _messages.MessageField('LabelsValue', 6)
  name = _messages.StringField(7)
  pythonVersion = _messages.StringField(8)
  state = _messages.EnumField('StateValueValuesEnum', 9)
  updateTime = _messages.StringField(10)


class WorkflowRun(_messages.Message):
  r"""WorkflowRun represents a single execution of a Workflow.

  Enums:
    StateValueValuesEnum: Output only. The latest known state of the workflow
      run.

  Messages:
    LabelsValue: Optional. User-defined labels for this workflow run. The
      initial label entries for a workflow run are inherited from the workflow
      from which it was instantiated. There can be no more than 64 labels
      entries. Entries of the labels map are UTF8 strings that comply with the
      following restrictions: * Keys must conform to regexp:
      \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
      [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
      constrained to be <= 128 bytes in size.

  Fields:
    endTime: Output only. The end time of the workflow run.
    intervalStartTime: Output only. The start of the time interval
      corresponding to this workflow run. Only set for SCHEDULED runs. This
      value is analagous to the Apache Airflow notion of `execution_date`; it
      refers to the left boundary of the scheduling interval.
    labels: Optional. User-defined labels for this workflow run. The initial
      label entries for a workflow run are inherited from the workflow from
      which it was instantiated. There can be no more than 64 labels entries.
      Entries of the labels map are UTF8 strings that comply with the
      following restrictions: * Keys must conform to regexp:
      \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
      [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
      constrained to be <= 128 bytes in size.
    name: Output only. The resource name of the workflow run, in the form: "pr
      ojects/{projectId}/locations/{locationId}/workflows/{workflowId}/runs/{r
      unId}"
    startTime: Output only. The start time of the workflow run.
    state: Output only. The latest known state of the workflow run.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The latest known state of the workflow run.

    Values:
      STATE_UNSPECIFIED: The state of the WorkflowRun is unknown.
      RUNNING: The WorkflowRun is currently running.
      SUCCESS: The WorkflowRun succeeded.
      FAILED: The WorkflowRun failed.
    """
    STATE_UNSPECIFIED = 0
    RUNNING = 1
    SUCCESS = 2
    FAILED = 3

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels for this workflow run. The initial label
    entries for a workflow run are inherited from the workflow from which it
    was instantiated. There can be no more than 64 labels entries. Entries of
    the labels map are UTF8 strings that comply with the following
    restrictions: * Keys must conform to regexp: \p{Ll}\p{Lo}{0,62} * Values
    must conform to regexp: [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values
    are additionally constrained to be <= 128 bytes in size.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  endTime = _messages.StringField(1)
  intervalStartTime = _messages.StringField(2)
  labels = _messages.MessageField('LabelsValue', 3)
  name = _messages.StringField(4)
  startTime = _messages.StringField(5)
  state = _messages.EnumField('StateValueValuesEnum', 6)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for interacting with the Cloud NetApp Files Backup Policy API resource."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.netapp import constants
from googlecloudsdk.api_lib.netapp import util as netapp_util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log
from googlecloudsdk.core import resources


class BackupPoliciesClient(object):
  """Wrapper for working with Backup Policies in the Cloud NetApp Files API Client."""

  def __init__(self, release_track=base.ReleaseTrack.BETA):
    if release_track == base.ReleaseTrack.BETA:
      self._adapter = BetaBackupPoliciesAdapter()
    elif release_track == base.ReleaseTrack.GA:
      self._adapter = BackupPoliciesAdapter()
    else:
      raise ValueError('[{}] is not a valid API version.'.format(
          netapp_util.VERSION_MAP[release_track]))

  @property
  def client(self):
    return self._adapter.client

  @property
  def messages(self):
    return self._adapter.messages

  def WaitForOperation(self, operation_ref):
    """Waits on the long-running operation until the done field is True.

    Args:
      operation_ref: the operation reference.

    Raises:
      waiter.OperationError: if the operation contains an error.

    Returns:
      the 'response' field of the Operation.
    """
    return waiter.WaitFor(
        waiter.CloudOperationPollerNoResources(
            self.client.projects_locations_operations
        ),
        operation_ref,
        'Waiting for [{0}] to finish'.format(operation_ref.Name()),
    )

  def CreateBackupPolicy(self, backuppolicy_ref, async_, backup_policy):
    """Create a Cloud NetApp Backup Policy."""
    request = self.messages.NetappProjectsLocationsBackupPoliciesCreateRequest(
        parent=backuppolicy_ref.Parent().RelativeName(),
        backupPolicyId=backuppolicy_ref.Name(),
        backupPolicy=backup_policy,
    )
    create_op = self.client.projects_locations_backupPolicies.Create(request)
    if async_:
      return create_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        create_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def ParseBackupPolicy(
      self,
      name=None,
      enabled=None,
      daily_backup_limit=None,
      weekly_backup_limit=None,
      monthly_backup_limit=None,
      description=None,
      labels=None
  ):
    """Parses the command line arguments for Create Backup Policy into a message.

    Args:
      name: the name of the Backup Policy
      enabled: the Boolean value indicating whether or not backups are made
        automatically according to schedule.
      daily_backup_limit: the number of daily backups to keep.
      weekly_backup_limit: the number of weekly backups to keep.
      monthly_backup_limit: the number of monthly backups to keep.
      description: the description of the Backup Policy.
      labels: the parsed labels value

    Returns:
      The configuration that will be used as the request body for creating a
      Cloud NetApp Backup Policy.
    """
    backup_policy = self.messages.BackupPolicy()
    backup_policy.name = name
    backup_policy.enabled = enabled
    backup_policy.dailyBackupLimit = daily_backup_limit
    backup_policy.weeklyBackupLimit = weekly_backup_limit
    backup_policy.monthlyBackupLimit = monthly_backup_limit
    backup_policy.description = description
    backup_policy.labels = labels
    return backup_policy

  def ListBackupPolicies(self, location_ref, limit=None):
    """Make API calls to List Cloud NetApp Backup Policies.

    Args:
      location_ref: The parsed location of the listed NetApp Backup Policies.
      limit: The number of Cloud NetApp Backup Policies
        to limit the results to. This limit is passed to
        the server and the server does the limiting.

    Returns:
      Generator that yields the Cloud NetApp Backup Policies.
    """
    request = self.messages.NetappProjectsLocationsBackupPoliciesListRequest(
        parent=location_ref)
    # Check for unreachable locations.
    response = self.client.projects_locations_backupPolicies.List(request)
    for location in response.unreachable:
      log.warning('Location {} may be unreachable.'.format(location))
    return list_pager.YieldFromList(
        self.client.projects_locations_backupPolicies,
        request,
        field=constants.BACKUP_POLICY_RESOURCE,
        limit=limit,
        batch_size_attribute='pageSize')

  def GetBackupPolicy(self, backuppolicy_ref):
    """Get Cloud NetApp Backup Policy information."""
    request = self.messages.NetappProjectsLocationsBackupPoliciesGetRequest(
        name=backuppolicy_ref.RelativeName())
    return self.client.projects_locations_backupPolicies.Get(request)

  def DeleteBackupPolicy(self, backuppolicy_ref, async_):
    """Deletes an existing Cloud NetApp Backup Policy."""
    request = (
        self.messages.NetappProjectsLocationsBackupPoliciesDeleteRequest(
            name=backuppolicy_ref.RelativeName()
        )
    )
    return self._DeleteBackupPolicy(async_, request)

  def _DeleteBackupPolicy(self, async_, request):
    delete_op = self.client.projects_locations_backupPolicies.Delete(request)
    if async_:
      return delete_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        delete_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def UpdateBackupPolicy(
      self, backuppolicy_ref, backup_policy, update_mask, async_
  ):
    """Updates a Backup Policy.

    Args:
      backuppolicy_ref: the reference to the Backup Policy.
      backup_policy: Backup Policy message, the updated Backup Policy.
      update_mask: str, a comma-separated list of updated fields.
      async_: bool, if False, wait for the operation to complete.
    Returns:
      an Operation or Backup Policy message.
    """
    update_op = self._adapter.UpdateBackupPolicy(
        backuppolicy_ref, backup_policy, update_mask
    )
    if async_:
      return update_op
    operation_ref = resources.REGISTRY.ParseRelativeName(
        update_op.name, collection=constants.OPERATIONS_COLLECTION
    )
    return self.WaitForOperation(operation_ref)

  def ParseUpdatedBackupPolicy(
      self,
      backup_policy,
      enabled=None,
      daily_backup_limit=None,
      weekly_backup_limit=None,
      monthly_backup_limit=None,
      description=None,
      labels=None,
  ):
    """Parses updates into an Backup Policy."""
    return self._adapter.ParseUpdatedBackupPolicy(
        backup_policy=backup_policy,
        enabled=enabled,
        daily_backup_limit=daily_backup_limit,
        weekly_backup_limit=weekly_backup_limit,
        monthly_backup_limit=monthly_backup_limit,
        description=description,
        labels=labels,
    )


class BackupPoliciesAdapter(object):
  """Adapter for the GA Cloud NetApp Files API for Backup Policies."""

  def __init__(self):
    self.release_track = base.ReleaseTrack.GA
    self.client = netapp_util.GetClientInstance(
        release_track=self.release_track
    )
    self.messages = netapp_util.GetMessagesModule(
        release_track=self.release_track
    )

  def ParseUpdatedBackupPolicy(
      self, backup_policy, daily_backup_limit=None, weekly_backup_limit=None,
      monthly_backup_limit=None, enabled=None, description=None, labels=None
  ):
    """Parses updates into a new Backup Policy."""
    if enabled is not None:
      backup_policy.enabled = enabled
    if daily_backup_limit is not None:
      backup_policy.dailyBackupLimit = daily_backup_limit
    if weekly_backup_limit is not None:
      backup_policy.weeklyBackupLimit = weekly_backup_limit
    if monthly_backup_limit is not None:
      backup_policy.monthlyBackupLimit = monthly_backup_limit
    if description is not None:
      backup_policy.description = description
    if labels is not None:
      backup_policy.labels = labels
    return backup_policy

  def UpdateBackupPolicy(self, backuppolicy_ref, backup_policy, update_mask):
    """Send a Patch request for the Cloud NetApp Backup Policy."""
    update_request = (
        self.messages.NetappProjectsLocationsBackupPoliciesPatchRequest(
            backupPolicy=backup_policy,
            name=backuppolicy_ref.RelativeName(),
            updateMask=update_mask))
    update_op = self.client.projects_locations_backupPolicies.Patch(
        update_request)
    return update_op


class BetaBackupPoliciesAdapter(BackupPoliciesAdapter):
  """Adapter for the Beta Cloud NetApp Files API for Backup Policies."""

  def __init__(self):
    super(BetaBackupPoliciesAdapter, self).__init__()
    self.release_track = base.ReleaseTrack.BETA
    self.client = netapp_util.GetClientInstance(
        release_track=self.release_track
    )
    self.messages = netapp_util.GetMessagesModule(
        release_track=self.release_track
    )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for general Cloud NetApp Files commands."""


from googlecloudsdk.api_lib.netapp import constants

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import properties

## Attribute configs ##


def GetLocationAttributeConfig():
  """Return the Location Attribute Config for resources.

  Returns:
    ResourceParameterAttributeConfig for location.
  """
  fallthroughs = [
      ## if location is not set, use value from
      ## gcloud config get-value netapp/location or netapp/region
      deps.ArgFallthrough('--location'),
      deps.PropertyFallthrough(properties.VALUES.netapp.location),
  ]

  return concepts.ResourceParameterAttributeConfig(
      name='location',
      fallthroughs=fallthroughs,
      help_text='The location of the {resource}.')


def GetStoragePoolAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'storage_pool', 'The instance of the {resource}.')


def GetVolumeAttributeConfig(positional=True):
  """Return the Volume Attribute Config for resources.

  Args:
    positional: boolean value describing whether volume attribute is conditional

  Returns:
    volume resource parameter attribute config for resource specs

  """
  if positional:
    fallthroughs = []
  else:
    ## For when we need a volume attribute in Snapshot resource spec, we want
    ## to assign fallthrough flags
    fallthroughs = [deps.ArgFallthrough('--volume')]
  help_text = ('The instance of the {resource}' if positional else (
      'The volume of the {resource}'))
  return concepts.ResourceParameterAttributeConfig(
      name='volume', fallthroughs=fallthroughs, help_text=help_text)


def GetSnapshotAttributeConfig(positional=True):
  if positional:
    help_text = 'The instance of the {resource}'
  else:
    help_text = 'The snapshot of the {resource}'
  return concepts.ResourceParameterAttributeConfig(
      'snapshot', help_text=help_text)


def GetReplicationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'replication', 'The instance of the {resource}')


def GetOperationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'operation', 'The Cloud NetApp operation.')


def GetActiveDirectoryAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'active_directory', 'The instance of the {resource}.'
  )


def GetBackupVaultAttributeConfig(positional=True):
  fallthroughs = []
  if not positional:
    ## For when we need a Backup Vault attribute in Backup resource spec
    ## we want to assign fallthrough flags
    fallthroughs = [deps.ArgFallthrough('--backup-vault')]
  return concepts.ResourceParameterAttributeConfig(
      'backup_vault', 'The Backup Vault of the {resource}.',
      fallthroughs=fallthroughs
  )


def GetBackupAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'backup', 'The instance of the {resource}.')


def GetBackupPolicyAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'backup_policy', 'The instance of the {resource}.'
  )


def GetKmsConfigAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'kms_config', 'The instance of the {resource}')


def GetKeyRingAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'key_ring', 'The instance of the {resource}.'
  )


def GetCryptoKeyAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'crypto_key', 'The instance of the {resource}.'
  )


def GetQuotaRuleAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'quota_rule', 'The instance of the {resource}.'
  )


def GetHostGroupAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      'host_group', 'The instance of the {resource}.'
  )


## Resource Specs ##


def GetLocationResourceSpec():
  location_attribute_config = GetLocationAttributeConfig()
  location_attribute_config.fallthroughs = []
  return concepts.ResourceSpec(
      constants.LOCATIONS_COLLECTION,
      resource_name='location',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=location_attribute_config)


def GetListingLocationResourceSpec():
  location_attribute_config = GetLocationAttributeConfig()
  location_attribute_config.fallthroughs.insert(
      0, deps.Fallthrough(lambda: '-', hint='uses all locations by default.'))
  return concepts.ResourceSpec(
      constants.LOCATIONS_COLLECTION,
      resource_name='location',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=location_attribute_config)


def GetOperationResourceSpec():
  return concepts.ResourceSpec(
      constants.OPERATIONS_COLLECTION,
      resource_name='operation',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      operationsId=GetOperationAttributeConfig())


def GetStoragePoolResourceSpec():
  return concepts.ResourceSpec(
      constants.STORAGEPOOLS_COLLECTION,
      resource_name='storage_pool',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      storagePoolsId=GetStoragePoolAttributeConfig())


def GetVolumeResourceSpec(positional=True):
  return concepts.ResourceSpec(
      constants.VOLUMES_COLLECTION,
      resource_name='volume',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      volumesId=GetVolumeAttributeConfig(positional=positional),
  )


def GetSnapshotResourceSpec(source_snapshot_op=False, positional=True):
  """Gets the Resource Spec for Snapshot.

  Args:
    source_snapshot_op: Boolean on whether operation uses snapshot as source or
      not.
    positional: Boolean on whether resource is positional arg ornot

  Returns:
    The Resource Spec for Snapshot
  """
  location_attribute_config = GetLocationAttributeConfig()
  volume_attribute_config = GetVolumeAttributeConfig(positional=False)
  if source_snapshot_op:
    # if revert op or backup op (create, update) that is a source snapshot op,
    # we don't want volume attribute to have any fallthroughs
    # (--volume arg) since volume is positional in revert.
    volume_attribute_config.fallthroughs = []
  if not positional:
    location_attribute_config.fallthroughs = [
        deps.PropertyFallthrough(properties.VALUES.netapp.location),
    ]
  return concepts.ResourceSpec(
      constants.SNAPSHOTS_COLLECTION,
      resource_name='snapshot',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=location_attribute_config,
      volumesId=volume_attribute_config,
      snapshotsId=GetSnapshotAttributeConfig(positional=positional))


def GetReplicationResourceSpec():
  location_attribute_config = GetLocationAttributeConfig()
  volume_attribute_config = GetVolumeAttributeConfig(positional=False)
  return concepts.ResourceSpec(
      constants.REPLICATIONS_COLLECTION,
      resource_name='replication',
      api_version=constants.BETA_API_VERSION,
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=location_attribute_config,
      volumesId=volume_attribute_config,
      replicationsId=GetReplicationAttributeConfig())


def GetActiveDirectoryResourceSpec():
  return concepts.ResourceSpec(
      constants.ACTIVEDIRECTORIES_COLLECTION,
      resource_name='active_directory',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      activeDirectoriesId=GetActiveDirectoryAttributeConfig())


def GetKmsConfigResourceSpec():
  return concepts.ResourceSpec(
      constants.KMSCONFIGS_COLLECTION,
      resource_name='kms_config',
      api_version=constants.BETA_API_VERSION,
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      kmsConfigsId=GetKmsConfigAttributeConfig())


def GetBackupVaultResourceSpec(positional=True):
  return concepts.ResourceSpec(
      constants.BACKUPVAULTS_COLLECTION,
      resource_name='backup_vault',
      api_version=constants.BETA_API_VERSION,
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      backupVaultsId=GetBackupVaultAttributeConfig(positional=positional),
  )


def GetBackupResourceSpec(positional=True):
  location_attribute_config = GetLocationAttributeConfig()
  backup_vault_attribute_config = GetBackupVaultAttributeConfig(
      positional=False
  )
  if not positional:
    location_attribute_config.fallthroughs = [
        deps.PropertyFallthrough(properties.VALUES.netapp.location),
    ]
  return concepts.ResourceSpec(
      constants.BACKUPS_COLLECTION,
      resource_name='backup',
      api_version=constants.BETA_API_VERSION,
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=location_attribute_config,
      backupVaultsId=backup_vault_attribute_config,
      backupsId=GetBackupAttributeConfig(),
  )


def GetBackupPolicyResourceSpec():
  return concepts.ResourceSpec(
      constants.BACKUPPOLICIES_COLLECTION,
      resource_name='backup_policy',
      api_version=constants.BETA_API_VERSION,
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      backupPoliciesId=GetBackupPolicyAttributeConfig())


def GetCryptoKeyResourceSpec():
  return concepts.ResourceSpec(
      'cloudkms.projects.locations.keyRings.cryptoKeys',
      resource_name='crypto_key',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      keyRingsId=GetKeyRingAttributeConfig(),
      cryptoKeysId=GetCryptoKeyAttributeConfig()
  )


def GetQuotaRuleResourceSpec():
  return concepts.ResourceSpec(
      constants.QUOTA_RULES_COLLECTION,
      resource_name='quota_rule',
      api_version=constants.BETA_API_VERSION,
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      volumesId=GetVolumeAttributeConfig(positional=False),
      quotaRulesId=GetQuotaRuleAttributeConfig(),
  )


def GetHostGroupResourceSpec():
  return concepts.ResourceSpec(
      constants.HOST_GROUPS_COLLECTION,
      resource_name='host_group',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=GetLocationAttributeConfig(),
      hostGroupsId=GetHostGroupAttributeConfig(),
  )

## Presentation Specs ##


def GetLocationPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'location', GetLocationResourceSpec(), group_help, required=True)


def GetResourceListingLocationPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      '--location',
      GetListingLocationResourceSpec(),
      group_help)


def GetOperationPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'operation', GetOperationResourceSpec(), group_help, required=True)


def GetStoragePoolPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'storage_pool', GetStoragePoolResourceSpec(), group_help, required=True)


def GetVolumePresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'volume', GetVolumeResourceSpec(), group_help, required=True)


def GetSnapshotPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'snapshot',
      GetSnapshotResourceSpec(),
      group_help,
      required=True,
      flag_name_overrides={'volume': ''})


def GetReplicationPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'replication',
      GetReplicationResourceSpec(),
      group_help,
      required=True,
      flag_name_overrides={'volume': ''})


def GetActiveDirectoryPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'active_directory',
      GetActiveDirectoryResourceSpec(),
      group_help,
      required=True)


def GetKmsConfigPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'kms_config',
      GetKmsConfigResourceSpec(),
      group_help,
      required=True)


def GetBackupVaultPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'backup_vault',
      GetBackupVaultResourceSpec(),
      group_help,
      required=True)


def GetBackupPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'backup',
      GetBackupResourceSpec(),
      group_help,
      required=True,
      flag_name_overrides={'backup_vault': ''})


def GetQuotaRulePresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'quota_rule',
      GetQuotaRuleResourceSpec(),
      group_help,
      required=True,
      flag_name_overrides={'volume': ''})


def GetHostGroupPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'host_group',
      GetHostGroupResourceSpec(),
      group_help,
      required=True,
  )


def GetOntapModeStoragePoolPresentationSpec(
    group_help: str,
    required: bool = True,
) -> presentation_specs.ResourcePresentationSpec:
  """Returns the ResourcePresentationSpec for a ONTAP-mode Storage Pool.

  Args:
    group_help: The help text for the argument group.
    required: Whether the argument is required.

  Returns:
    A ResourcePresentationSpec for an ONTAP-mode Storage Pool.
  """

  return presentation_specs.ResourcePresentationSpec(
      'storage_pool',
      GetStoragePoolResourceSpec(),
      group_help,
      required=required,
      prefixes=False,
  )


# TODO(b/290375665): Add more unit tests to test Backup Poicy
# Presentation, resource specs and flags
def GetBackupPolicyPresentationSpec(group_help):
  return presentation_specs.ResourcePresentationSpec(
      'backup_policy',
      GetBackupPolicyResourceSpec(),
      group_help,
      required=True)

## Add args to arg parser ##


def AddResourceDescriptionArg(parser, resource_name):
  """Add Description arg to arg_parser for a resource called resource_name."""
  parser.add_argument(
      '--description',
      required=False,
      help='A description of the Cloud NetApp {}'.format(resource_name))


def AddResourceCapacityArg(parser, resource_name, required=True):
  """Add Capacity arg to arg_parser for a resource called resource_name."""
  parser.add_argument(
      '--capacity',
      type=arg_parsers.BinarySize(
          default_unit='GiB', suggested_binary_size_scales=['GiB', 'TiB']),
      required=required,
      help=('The desired capacity of the {} in GiB or TiB units.'
            'If no capacity unit is specified, GiB is assumed.'.format(
                resource_name)))


def AddResourceAsyncFlag(parser):
  help_text = """Return immediately, without waiting for the operation
  in progress to complete."""
  concepts.ResourceParameterAttributeConfig(name='async', help_text=help_text)
  base.ASYNC_FLAG.AddToParser(parser)


def AddResourcePeerClusterNameArg(parser, required=True):
  """Adds the Peer Cluster Name (--peer-cluster-name) arg to the given parser.

  Args:
    parser: Argparse parser.
    required: Required to establish both cluster and svm peering.
  """
  parser.add_argument(
      '--peer-cluster-name',
      type=str,
      required=required,
      help="""Name of the destination cluster to be peered
        with the source cluster.""",
  )


def AddResourcePeerSvmNameArg(parser, required=True):
  """Adds the Peer SVM Name (--peer-svm-name) arg to the given parser.

  Args:
    parser: Argparse parser.
    required: Required to establish both cluster and svm peering.
  """
  parser.add_argument(
      '--peer-svm-name',
      type=str,
      required=required,
      help="""Name of the local source vserver svm to be peered
        with the destination cluster.""",
  )


def AddResourcePeerVolumeNameArg(parser, required=True):
  """Adds the Peer Volume Name (--peer-volume-name) arg to the given parser.

  Args:
    parser: Argparse parser.
    required: Required to establish both cluster and svm peering.
  """
  parser.add_argument(
      '--peer-volume-name',
      type=str,
      required=required,
      help="""Name of the source volume to be peered
       with the destination volume.""",
  )


def AddResourcePeerIpAddressesArg(parser):
  """Adds the Peer IP Addresses (--peer-ip-addresses) arg to the given parser.

  Args:
    parser: Argparse parser.

  Not required for svm peering.
  """
  parser.add_argument(
      '--peer-ip-addresses',
      type=arg_parsers.ArgList(min_length=1, element_type=str),
      metavar='PEER_IP_ADDRESS',
      help=(
          'List of ip addresses to be used for peering. This is required for'
          ' cluster peering, not required for svm peering.'
      ),
  )



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List command for Orchestration Pipelines Resource Types."""

from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.command_lib.orchestration_pipelines.handlers import registry


@calliope_base.Hidden
@calliope_base.DefaultUniverseOnly
@calliope_base.ReleaseTracks(calliope_base.ReleaseTrack.BETA)
class List(calliope_base.ListCommand):
  """List supported orchestration pipelines resource types."""

  detailed_help = {
      "DESCRIPTION": (
          "List all resource types supported by orchestration pipelines "
          "deployments."
      ),
      "EXAMPLES": (
          """
        To list all supported resource types, run:

          $ {command}

        To discover a specific resource type and its properties, run:

          $ {command} --filter="type=bigquery.table"

        To output the list in JSON format, run:

          $ {command} --format=json
      """
      ),
  }

  @staticmethod
  def Args(parser):
    """Add arguments to the parser."""
    parser.display_info.AddFormat("""
        table(
            type:label=TYPE,
            description:label=DESCRIPTION,
            documentation_uri:label=DOCUMENTATION_URI
        )
    """)

  def Run(self, args):
    """Run the list command."""
    results = []
    for type_name, handler_class in registry.GetHandlerClasses().items():
      # Instantiate with dummy values to read properties
      dummy_resource = type(
          "DummyResource",
          (),
          {
              "type": type_name,
              "name": "dummy",
              "definition": {},
              "metadata": None,
              "api_version": None,
          },
      )()
      dummy_environment = type(
          "DummyEnvironment", (), {"project": "dummy", "region": "dummy"}
      )()

      # Bypass __init__ to avoid slow API client initialization and auth
      handler_instance = handler_class.__new__(handler_class)
      handler_instance.resource = dummy_resource
      handler_instance.environment = dummy_environment

      api_version = handler_instance._api_version  # pylint: disable=protected-access
      parent_type = handler_instance.get_parent_resource_type()
      doc_url = handler_instance.get_documentation_uri()

      description = handler_class.description

      results.append({
          "type": type_name,
          "parent_type": parent_type,
          "api_version": api_version,
          "description": description,
          "documentation_uri": doc_url,
      })

    # Sort results by resource type for predictable output
    return sorted(results, key=lambda x: x["type"])



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Apphub Applications API."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.apphub import consts as api_lib_consts
from googlecloudsdk.api_lib.apphub import utils as api_lib_utils
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util


class ApplicationsClient(object):
  """Client for Applications in apphub API."""

  def __init__(self, release_track=base.ReleaseTrack.ALPHA):
    self.client = api_lib_utils.GetClientInstance(release_track)
    self.messages = api_lib_utils.GetMessagesModule(release_track)
    self._app_client = self.client.projects_locations_applications
    self._lookup_client = self.client.projects_locations
    self._poller = waiter.CloudOperationPoller(
        self._app_client,
        self.client.projects_locations_operations,
    )
    self._delete_poller = waiter.CloudOperationPollerNoResources(
        self.client.projects_locations_operations,
    )
    self._lookup_poller = waiter.CloudOperationPoller(
        self._lookup_client,
        self.client.projects_locations_operations,
    )

  def Describe(self, app_id):
    """Describe an application in the Project/location.

    Args:
      app_id: str, the application id.

    Returns:
      Described service project Resource.
    """
    describe_req = self.messages.ApphubProjectsLocationsApplicationsGetRequest(
        name=app_id
    )
    return self._app_client.Get(describe_req)

  def List(
      self,
      parent,
      limit=None,
      page_size=100,
  ):
    """List applications in the Projects/Location.

    Args:
      parent: str, projects/{projectId}/locations/{location}
      limit: int or None, the total number of results to return. Default value
        is None
      page_size: int, the number of entries in each batch (affects requests
        made, but not the yielded results). Default value is 100.

    Returns:
      Generator of matching service projects.
    """
    list_req = self.messages.ApphubProjectsLocationsApplicationsListRequest(
        parent=parent
    )
    return list_pager.YieldFromList(
        self._app_client,
        list_req,
        field='applications',
        batch_size=page_size,
        limit=limit,
        batch_size_attribute='pageSize',
    )

  def Create(
      self,
      app_id,
      scope_type,
      display_name,
      description,
      attributes,
      async_flag,
      parent,
  ):
    """Creates an application in the Project/location.

    Args:
      app_id: str, Application ID
      scope_type: str, Scope of the Application
      display_name: str, Human-friendly display name
      description: str, Description of the Application
      attributes: Attributes, Attributes of the Application
      async_flag: Boolean value for async operation. If true the operation will
        be async
      parent: parent for project resource

    Returns:
      Application or Operation based on async flag value.
    """
    application = self.messages.Application(
        description=description,
        displayName=display_name,
        scope=self.messages.Scope(
            type=self.messages.Scope.TypeValueValuesEnum(scope_type)
        ),
        attributes=attributes,
    )

    create_req = self.messages.ApphubProjectsLocationsApplicationsCreateRequest(
        application=application, applicationId=app_id, parent=parent
    )
    operation = self._app_client.Create(create_req)

    if async_flag:
      return operation

    create_response = api_lib_utils.WaitForOperation(
        self._poller,
        operation,
        api_lib_consts.CreateApplication.WAIT_FOR_ADD_MESSAGE,
        api_lib_consts.CreateApplication.ADD_TIMELIMIT_SEC,
    )

    return create_response

  def Update(self, args, app_ref, release_track=base.ReleaseTrack.ALPHA):
    """Updates an Apphub application.

    Args:
      args: args, Arguments provided by the client
      app_ref: Application reference
      release_track: release_track provided by the client

    Returns:
      Application or Operation based on async flag value.
    """

    update_mask = ''

    attributes = api_lib_utils.GetMessagesModule(release_track).Attributes()
    application = self.messages.Application(attributes=attributes)

    if args.environment_type is not None:
      attributes.environment = api_lib_utils.GetMessagesModule(
          release_track
      ).Environment(
          type=api_lib_utils.GetMessagesModule(
              release_track
          ).Environment.TypeValueValuesEnum(args.environment_type)
      )
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplication.UPDATE_MASK_ENVIRONMENT_FIELD_NAME,
      )

    if args.criticality_type:
      attributes.criticality = api_lib_utils.GetMessagesModule(
          release_track
      ).Criticality(
          type=api_lib_utils.GetMessagesModule(
              release_track
          ).Criticality.TypeValueValuesEnum(args.criticality_type)
      )
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplication.UPDATE_MASK_CRITICALITY_FIELD_NAME,
      )

    for b_owner in args.business_owners or []:
      business_owner = api_lib_utils.GetMessagesModule(
          release_track
      ).ContactInfo()
      business_owner.email = b_owner.get('email', None)
      if b_owner.get('display-name', None):
        business_owner.displayName = b_owner.get('display-name', None)
      if release_track == base.ReleaseTrack.ALPHA:
        if b_owner.get('channel-uri', None):
          business_owner.channel = api_lib_utils.GetMessagesModule(
              release_track
          ).Channel(uri=b_owner.get('channel-uri'))
      attributes.businessOwners.append(business_owner)
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplication.UPDATE_MASK_BUSINESS_OWNERS_FIELD_NAME,
      )

    for d_owner in args.developer_owners or []:
      developer_owner = api_lib_utils.GetMessagesModule(
          release_track
      ).ContactInfo()
      developer_owner.email = d_owner.get('email', None)
      if d_owner.get('display-name', None):
        developer_owner.displayName = d_owner.get('display-name', None)
      if release_track == base.ReleaseTrack.ALPHA:
        if d_owner.get('channel-uri', None):
          developer_owner.channel = api_lib_utils.GetMessagesModule(
              release_track
          ).Channel(uri=d_owner.get('channel-uri'))
      attributes.developerOwners.append(developer_owner)
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplication.UPDATE_MASK_DEVELOPER_OWNERS_FIELD_NAME,
      )

    for o_owner in args.operator_owners or []:
      operator_owner = api_lib_utils.GetMessagesModule(
          release_track
      ).ContactInfo()
      operator_owner.email = o_owner.get('email', None)
      if o_owner.get('display-name'):
        operator_owner.displayName = o_owner.get('display-name')
      if release_track == base.ReleaseTrack.ALPHA:
        if o_owner.get('channel-uri'):
          operator_owner.channel = api_lib_utils.GetMessagesModule(
              release_track
          ).Channel(uri=o_owner.get('channel-uri'))
      attributes.operatorOwners.append(operator_owner)
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplication.UPDATE_MASK_OPERATOR_OWNERS_FIELD_NAME,
      )

    if args.display_name is not None:
      application.displayName = args.display_name
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplication.UPDATE_MASK_DISPLAY_NAME_FIELD_NAME,
      )

    if args.description is not None:
      application.description = args.description
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplication.UPDATE_MASK_DESCRIPTION_FIELD_NAME,
      )

    patch_req = self.messages.ApphubProjectsLocationsApplicationsPatchRequest(
        application=application,
        name=app_ref.RelativeName(),
        updateMask=update_mask,
    )

    operation = self._app_client.Patch(patch_req)

    if args.async_:
      return operation

    patch_response = api_lib_utils.WaitForOperation(
        self._poller,
        operation,
        api_lib_consts.UpdateApplication.WAIT_FOR_UPDATE_MESSAGE,
        api_lib_consts.UpdateApplication.ADD_TIMELIMIT_SEC,
    )

    return patch_response

  def Delete(self, app_id, async_flag):
    """Delete an application in the Project/location.

    Args:
      app_id: str, The name for the application being deleted
      async_flag: Boolean value for async operation. If true the operation will
        be async

    Returns:
      Empty Response Message or Operation based on async flag value.
    """
    remove_req = self.messages.ApphubProjectsLocationsApplicationsDeleteRequest(
        name=app_id
    )
    operation = self._app_client.Delete(remove_req)

    if async_flag:
      return operation

    delete_response = api_lib_utils.WaitForOperation(
        self._delete_poller,
        operation,
        api_lib_consts.DeleteApplication.WAIT_FOR_DELETE_MESSAGE,
        api_lib_consts.DeleteApplication.REMOVE_TIMELIMIT_SEC,
    )

    return delete_response

  def GetIamPolicy(self, app_id):
    """Fetch the IAM Policy attached to the sepcified application.

    Args:
      app_id: str, the application id.

    Returns:
      The application's IAM Policy.
    """
    # version = iam_util.MAX_LIBRARY_IAM_SUPPORTED_VERSION
    get_req = (
        self.messages.ApphubProjectsLocationsApplicationsGetIamPolicyRequest(
            resource=app_id,
        )
    )
    return self._app_client.GetIamPolicy(get_req)

  def SetIamPolicy(self, app_id, policy_file):
    """Sets an application's IamPolicy to the one provided.

    If 'policy_file' has no etag specified, this will BLINDLY OVERWRITE the IAM
    policy!

    Args:
        app_id: str, the application id..
        policy_file: a policy file.

    Returns:
        The IAM Policy.
    """
    policy = iam_util.ParsePolicyFile(policy_file, self.messages.Policy)
    return self._SetIamPolicyHelper(app_id, policy)

  def _SetIamPolicyHelper(self, app_id, policy):
    set_req = (
        self.messages.ApphubProjectsLocationsApplicationsSetIamPolicyRequest(
            resource=app_id,
            setIamPolicyRequest=self.messages.SetIamPolicyRequest(
                policy=policy,),
        ))
    return self._app_client.SetIamPolicy(set_req)

  def AddIamPolicyBinding(self, app_id, member, role):
    """Does an atomic Read-Modify-Write, adding the member to the role.

    Args:
        app_id: str, the application id.
        member: str, the principal to add the binding for.
        role: predefined role, the role name to assign to the principal.

    Returns:
        The IAM Policy.

    """
    policy = self.GetIamPolicy(app_id)
    iam_util.AddBindingToIamPolicy(self.messages.Binding, policy, member, role)
    return self._SetIamPolicyHelper(app_id, policy)

  def RemoveIamPolicyBinding(self, app_id, member, role):
    """Does an atomic Read-Modify-Write, removing the member to the role.

    Args:
        app_id: str, the application id.
        member: str, the principal to add the binding for.
        role: predefined role, the role name to assign to the principal.

    Returns:
        The IAM Policy.

    """
    policy = self.GetIamPolicy(app_id)
    iam_util.RemoveBindingFromIamPolicy(policy, member, role)
    return self._SetIamPolicyHelper(app_id, policy)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility functions to call Security Posture API."""


import datetime

from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.core import resources


API_NAME = 'securityposture'
VERSION_MAP = {
    base.ReleaseTrack.ALPHA: 'v1alpha',
    base.ReleaseTrack.GA: 'v1'
}


def GetMessagesModule(release_track=base.ReleaseTrack.ALPHA):
  api_version = VERSION_MAP.get(release_track)
  return apis.GetMessagesModule(API_NAME, api_version)


def GetClientInstance(release_track=base.ReleaseTrack.ALPHA):
  api_version = VERSION_MAP.get(release_track)
  return apis.GetClientInstance(API_NAME, api_version)


def GetOperationsRef(operation_id, release_track=base.ReleaseTrack.ALPHA):
  """Operations to Resource used for `waiter.WaitFor`.

  Args:
    operation_id: The operation ID for which resource reference is required.
    release_track: The release track to use for the API version.

  Returns:
    The resource reference for the operation.
  """
  return resources.REGISTRY.ParseRelativeName(
      operation_id,
      collection='securityposture.organizations.locations.operations',
      api_version=VERSION_MAP.get(release_track),
  )


def WaitForOperation(
    operation_ref,
    message,
    has_result=False,
    release_track=base.ReleaseTrack.ALPHA,
    max_wait=datetime.timedelta(seconds=600),
):
  """Waits for an operation to complete.

  Polls the Security Posture Operations service until the operation completes,
  fails, or max_wait_seconds elapses.

  Args:
    operation_ref: A Resource created by GetOperationRef describing the
      Operation.
    message: The message to display to the user while they wait.
    has_result: If True, the function will return the target of the operation
      when it completes. If False, nothing will be returned.
    release_track: The release track to use for the API version.
    max_wait: The time to wait for the operation to succeed before timing out.

  Returns:
    if has_result = True, a RemediationIntent entity.
    Otherwise, None.
  """
  client = GetClientInstance(release_track)
  resource_client = client.organizations_locations_remediationIntents
  operations_client = client.organizations_locations_operations
  if has_result:
    poller = waiter.CloudOperationPoller(
        resource_client, operations_client
    )
  else:
    # For no result expectations, just operations service client is required.
    poller = waiter.CloudOperationPollerNoResources(operations_client)

  response = waiter.WaitFor(
      poller, operation_ref, message, max_wait_ms=max_wait.seconds * 1000
  )
  return response



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing reservation blocks."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.reservations import resource_args
from googlecloudsdk.command_lib.compute.reservations.blocks import flags


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Describe(base.DescribeCommand):
  """Describe a Compute Engine reservation block."""

  @staticmethod
  def Args(parser):
    Describe.ReservationArg = (
        resource_args.GetReservationResourceArg()
    )
    Describe.ReservationArg.AddArgument(parser, operation_type='describe')
    flags.AddDescribeFlags(parser)
    flags.AddFullViewFlag(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    reservation_ref = Describe.ReservationArg.ResolveAsResource(
        args,
        holder.resources,
        default_scope=compute_scope.ScopeEnum.ZONE,
        scope_lister=compute_flags.GetDefaultScopeLister(client))

    view_enum = self.GetViewEnum(client, args)

    request = client.messages.ComputeReservationBlocksGetRequest(
        reservation=reservation_ref.reservation,
        zone=reservation_ref.zone,
        project=reservation_ref.project,
        reservationBlock=args.block_name,
        view=view_enum,
    )

    return client.MakeRequests([(client.apitools_client.reservationBlocks,
                                 'Get', request)])[0]

  def GetViewEnum(self, client, args):
    view_enum = None
    if args.IsSpecified('full_view'):
      if args.full_view == 'BLOCK_VIEW_FULL':
        view_enum = (
            client.messages.ComputeReservationBlocksGetRequest.ViewValueValuesEnum.FULL
        )
      if args.full_view == 'BLOCK_VIEW_BASIC':
        view_enum = (
            client.messages.ComputeReservationBlocksGetRequest.ViewValueValuesEnum.BASIC
        )
    return view_enum


Describe.detailed_help = {
    'EXAMPLES':
        """\
    To describe a reservation block in reservation my-reservation in my-zone
    with block name my-reservation-block-0001, run:

      $ {command} my-reservation --zone=my-zone --block-name=my-reservation-block-0001
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting values overridden in all-instances config."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import managed_instance_groups_utils as mig_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.instance_groups import flags as instance_groups_flags
from googlecloudsdk.command_lib.compute.instance_groups.managed import flags as managed_instance_groups_flags


@base.ReleaseTracks(
    base.ReleaseTrack.GA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.PREVIEW,
)
class Delete(base.DeleteCommand):
  """Delete all-instances-config of a managed instance group."""

  @classmethod
  def Args(cls, parser):
    instance_groups_flags.GetInstanceGroupManagerArg(
        region_flag=True).AddArgument(
            parser, operation_type='delete the all instances configuration for')
    managed_instance_groups_flags.AddFlagsForDeleteAllInstancesConfig(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    resources = holder.resources

    igm_ref = (instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG
               .ResolveAsResource)(
                   args,
                   resources,
                   scope_lister=compute_flags.GetDefaultScopeLister(client),
               )

    if igm_ref.Collection() not in [
        'compute.instanceGroupManagers', 'compute.regionInstanceGroupManagers'
    ]:
      raise ValueError('Unknown reference type {0}'.format(
          igm_ref.Collection()))

    patch_instance_group_manager = self._CreateInstanceGroupManagerPatch(
        args, client)
    return Delete._MakePatchRequest(client, igm_ref,
                                    patch_instance_group_manager)

  def _CreateInstanceGroupManagerPatch(self, args, client):
    """Creates IGM resource patch."""
    mig_utils.RegisterCustomInstancePropertiesPatchEncoders(client)

    metadata = args.metadata or []
    labels = args.labels or []
    return client.messages.InstanceGroupManager(
        allInstancesConfig=client.messages
        .InstanceGroupManagerAllInstancesConfig(
            properties=client.messages.InstancePropertiesPatch(
                metadata=client.messages.InstancePropertiesPatch.MetadataValue(
                    additionalProperties=[
                        client.messages.InstancePropertiesPatch.MetadataValue
                        .AdditionalProperty(key=key, value=None)
                        for key in metadata
                    ]),
                labels=client.messages.InstancePropertiesPatch.LabelsValue(
                    additionalProperties=[
                        client.messages.InstancePropertiesPatch.LabelsValue
                        .AdditionalProperty(key=key, value=None)
                        for key in labels
                    ]))))

  @staticmethod
  def _MakePatchRequest(client, igm_ref, igm_updated_resource):
    if igm_ref.Collection() == 'compute.instanceGroupManagers':
      service = client.apitools_client.instanceGroupManagers
      request = client.messages.ComputeInstanceGroupManagersPatchRequest(
          instanceGroupManager=igm_ref.Name(),
          instanceGroupManagerResource=igm_updated_resource,
          project=igm_ref.project,
          zone=igm_ref.zone)
    else:
      service = client.apitools_client.regionInstanceGroupManagers
      request = client.messages.ComputeRegionInstanceGroupManagersPatchRequest(
          instanceGroupManager=igm_ref.Name(),
          instanceGroupManagerResource=igm_updated_resource,
          project=igm_ref.project,
          region=igm_ref.region)
    return client.MakeRequests([(service, 'Patch', request)])


Delete.detailed_help = {
    'brief':
        'Delete values defined in the all-instances configuration of a managed '
        'instance group.',
    'DESCRIPTION':
        """\
        *{command}* deletes one or more values defined in the all-instances
        configuration of a managed instance group.

        To apply a revised all-instances configuration to existing instances
        in the group, use one of the following methods:

        - Update instances using the `update-instances` command.
        - Recreate instances using the `recreate-instances` command.
        - Use the `rolling-action start-update` command.
        - Use the API to set the group's `updatePolicy.type` to `PROACTIVE`.
        """,
    'EXAMPLES':
        """\
        To delete the group's all-instances configuration in order to stop
        overriding the group's instance template for a label with the key
        `label-key` and metadata with the key `metadata-key` in group
        `my-group`, run:

          $ {command} my-group
            --metadata=metadata-key
            --labels=label-key
        """
}



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API Library for gcloud scheduler CMEK config."""


from googlecloudsdk.core import exceptions


class RequiredFieldsMissingError(exceptions.Error):
  """Error for when calling a method when a required field is unspecified."""


class CmekConfig(object):
  """API client for Cloud Scheduler CMEK Config."""

  def __init__(self, messages, cmek_config_service):
    self.messages = messages
    self.cmek_config_service = cmek_config_service

  def GetCmekConfig(self, project_id, location_id):
    """Prepares and sends a GetCmekConfig request for the given CmekConfig."""
    cmek_config_name = (
        'projects/{project_id}/locations/{location_id}/cmekConfig'.format(
            project_id=project_id, location_id=location_id
        )
    )
    request = self.messages.CloudschedulerProjectsLocationsGetCmekConfigRequest(
        name=cmek_config_name,
    )

    return self.cmek_config_service.GetCmekConfig(request)

  def UpdateCmekConfig(self, project_id, location_id, cmek_config):
    """Prepares and sends an UpdateCmekConfig request for the given CmekConfig."""
    cmek_config_name = (
        'projects/{project_id}/locations/{location_id}/cmekConfig'.format(
            project_id=project_id, location_id=location_id
        )
    )
    request = (
        self.messages.CloudschedulerProjectsLocationsUpdateCmekConfigRequest(
            name=cmek_config_name, cmekConfig=cmek_config
        )
    )
    return self.cmek_config_service.UpdateCmekConfig(request)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List Artifact Registry resource locations."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.artifacts import util


DEFAULT_LIST_FORMAT = """\
    table(
      name:label=LOCATIONS
    )"""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List all Artifact Registry supported locations."""

  detailed_help = {
      "DESCRIPTION":
          "{description}",
      "EXAMPLES":
          """\
    To list all supported locations:

        $ {command}
    """,
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat(DEFAULT_LIST_FORMAT)
    base.URI_FLAG.RemoveFromParser(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided
        to this command invocation.

    Returns:
      A list of all supported locations.
    """

    return [{"name": loc} for loc in util.GetLocationList(args)]



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for Robin CNS related commands."""


from googlecloudsdk.api_lib.edge_cloud.container import util
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.core.console import console_io


def PromptToEnableRobinCNSConfirmation():
  """Displays warning and prompts user for confirmation."""
  message = """WARNING:
Enabling Robin CNS is irreversible. Once enabled, it cannot be disabled.
Enabling Robin CNS will take over all unused local Persistent Volumes (PVs)
in the cluster. Any data on these PVs will be permanently lost."""

  console_io.PromptContinue(message=message, default=False, cancel_on_no=True)


def EnableRobinCNSInRequest(req, args):
  """Set Robin CNS config in the cluster request message."""
  release_track = args.calliope_command.ReleaseTrack()
  messages = util.GetMessagesModule(release_track)
  if req.cluster is None:
    req.cluster = messages.Cluster()

  if req.cluster.systemAddonsConfig is None:
    req.cluster.systemAddonsConfig = messages.SystemAddonsConfig()
  if req.cluster.systemAddonsConfig.robinCloudNativeStorage is None:
    req.cluster.systemAddonsConfig.robinCloudNativeStorage = (
        messages.RobinCloudNativeStorage()
    )
  req.cluster.systemAddonsConfig.robinCloudNativeStorage.enable = True


def HandleEnableRobinCNSUpdate(ref, args, request):
  """Handles the --enable-robin-cns flag for UPDATE requests."""

  del ref

  if not flags.FlagIsExplicitlySet(args, "enable_robin_cns"):
    return request

  PromptToEnableRobinCNSConfirmation()
  EnableRobinCNSInRequest(request, args)

  _AddFieldToUpdateMask(
      "system_addons_config.robin_cloud_native_storage.enable", request
  )
  return request


def _AddFieldToUpdateMask(field, request):
  if not request.updateMask:
    request.updateMask = field
    return request

  if field not in request.updateMask:
    request.updateMask = request.updateMask + "," + field
  return request



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group for Artifact Registry container images."""


from googlecloudsdk.calliope import base


class Images(base.Group):
  """Manage Artifact Registry container images.

  To list images under the current project, repository, and location:

      $ {command} list

  To list images under repository `my-repo`, project `my-project`, in
  `us-central1`:

      $ {command} list us-central1-docker.pkg.dev/my-project/my-repo

  To list images with tags, under repository `my-repo`, project `my-project`
  across all locations:

      $ {command} list docker.pkg.dev/my-project/my-repo --include-tags

  To list all images under image `busy-box`, in repository `my-repo`, project
  `my-project` across all locations:

      $ {command} list docker.pkg.dev/my-project/my-repo/busy-box

  To delete image `busy-box` in `us-west1` and all of its digests and tags:

      $ {command} delete
      us-west1-docker.pkg.dev/my-project/my-repository/busy-box

  To delete image digest `abcxyz` under image `busy-box`:

      $ {command} delete
      us-west1-docker.pkg.dev/my-project/my-repository/busy-box@sha256:abcxyz

  To delete image digest `abcxyz` under image `busy-box` while there're some
  other tags associate with the digest:

      $ {command} delete
      us-west1-docker.pkg.dev/my-project/my-repository/busy-box@sha256:abcxyz
      --delete-tags

  To delete an image digest and its only tag `my-tag` under image `busy-box`:

      $ {command} delete
      us-west1-docker.pkg.dev/my-project/my-repository/busy-box:my-tag
  """

  category = base.CI_CD_CATEGORY



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for updating the KMS key of a persistent snapshot."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import kms_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.snapshots import flags as snapshots_flags

DETAILED_HELP = {
    'brief': 'Update the KMS key of a Compute Engine snapshot.',
    'DESCRIPTION': """
        * {command}* updates the KMS key of a Compute Engine persistent snapshot
        by rotating it to the primary version of the key or to the primary
        version of a new KMS key.
    """,
    'EXAMPLES': """
        To update the KMS key of a snapshot named example-snapshot-1, run:

          $ {command} example-snapshot-1

        To update the KMS key of a snapshot named example-snapshot-2 to a new
        KMS key named example-key in a key ring named example-key-ring in the
        global scope, run:

          $ {command} example-snapshot-2 --kms-key=example-key --kms-keyring=example-key-ring --kms-location=global
    """,
}


def _CommonArgs(parser):
  """Add arguments used for parsing in all command tracks."""
  snapshots_flags.MakeSnapshotArg(plural=False).AddArgument(
      parser, operation_type='update the KMS key for'
  )
  snapshots_flags.AddKmsKeyArg(parser)
  parser.add_argument(
      '--region',
      help='The region of the snapshot to update.',
      required=False,
  )


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.UniverseCompatible
class UpdateKmsKey(base.Command):
  """Update the KMS key of a persistent snapshot."""

  @classmethod
  def Args(cls, parser):
    _CommonArgs(parser)

  @classmethod
  def _GetApiHolder(cls, no_http=False):
    return base_classes.ComputeApiHolder(cls.ReleaseTrack(), no_http)

  def Run(self, args):
    compute_holder = self._GetApiHolder()
    client = compute_holder.client
    messages = client.messages

    kms_key_ref = kms_utils.MaybeGetKmsKey(args, messages, None)
    kms_key_name = kms_key_ref.kmsKeyName if kms_key_ref else None

    if args.region:
      snapshot_ref = snapshots_flags.MakeSnapshotArgForRegionalSnapshots(
          plural=False
      ).ResolveAsResource(args, compute_holder.resources)
    else:
      snapshot_ref = snapshots_flags.MakeSnapshotArg(
          plural=False
      ).ResolveAsResource(args, compute_holder.resources)

    if snapshot_ref.Collection() == 'compute.snapshots':
      service = client.apitools_client.snapshots
      request = messages.ComputeSnapshotsUpdateKmsKeyRequest(
          project=snapshot_ref.project,
          snapshot=snapshot_ref.Name(),
          snapshotUpdateKmsKeyRequest=messages.SnapshotUpdateKmsKeyRequest(
              kmsKeyName=kms_key_name
          ),
      )
      return client.MakeRequests([(service, 'UpdateKmsKey', request)])
    elif snapshot_ref.Collection() == 'compute.regionSnapshots':
      service = client.apitools_client.regionSnapshots
      request = messages.ComputeRegionSnapshotsUpdateKmsKeyRequest(
          project=snapshot_ref.project,
          region=snapshot_ref.region,
          snapshot=snapshot_ref.Name(),
          regionSnapshotUpdateKmsKeyRequest=messages.RegionSnapshotUpdateKmsKeyRequest(
              kmsKeyName=kms_key_name
          ),
      )
      return client.MakeRequests([(service, 'UpdateKmsKey', request)])


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class UpdateKmsKeyBeta(UpdateKmsKey):
  """Update the KMS key of a persistent snapshot."""


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateKmsKeyAlpha(UpdateKmsKeyBeta):
  """Update the KMS key of a persistent snapshot."""


UpdateKmsKey.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Custom printer for gsutil-style JSON."""


import json

from googlecloudsdk.core.resource import resource_printer
from googlecloudsdk.core.resource import resource_printer_base

_PRINTER_FORMAT = 'gsutiljson'


class GsutilJsonPrinter(resource_printer_base.ResourcePrinter):
  """Prints resource records as single line JSON string, just like gsutil.

  To use this resource printer, first call this class's Register() method in a
  target command's Args() method to add it to the available formatters. Then,
  use `gsutiljson[empty="Message"]` rather than the usual `json` formatter to
  mimic gsutil JSON output.

  Printer attributes:
    empty: Returns this value if the resource is empty or if `key` is missing.
        Defaults to printing ''.
    key: The key of the record to output. Only recommended when printing a
        single resource. The full record is printed by default.
    empty_prefix_key: The key of the record to use as a prefix for the `empty`
        string when the `key` attribute is specified and lacks a value.

  Attributes:
    _empty: True if no records were output.
    _delimiter: Delimiter string before the next record.
  """

  _BEGIN_DELIMITER = '['
  _END_DELIMITER = ']'

  def __init__(self, *args, **kwargs):
    super(GsutilJsonPrinter, self).__init__(
        *args, retain_none_values=True, **kwargs
    )
    self._empty = True
    self._delimiter = self._BEGIN_DELIMITER
    self._prefix = ''

  @staticmethod
  def Register():
    """Register this as a custom resource printer."""
    resource_printer.RegisterFormatter(
        _PRINTER_FORMAT, GsutilJsonPrinter
    )

  def _AddRecord(self, record, delimit=True):
    """Prints one element of a JSON-serializable Python object resource list.

    Allows intermingled delimit=True and delimit=False.

    Args:
      record: A JSON-serializable object.
      delimit: Dump one record if False, used by PrintSingleRecord().
    """
    element = record
    if 'key' in self.attributes:
      key = self.attributes['key']
      element = element.get(key, '')
    if not element:
      if 'empty_prefix_key' in self.attributes:
        self._prefix = str(record.get(self.attributes['empty_prefix_key'], ''))
      # ResourcePrinter.AddRecord() sets self._empty to False before calling
      # this function, so we remedy that here.
      self._empty = True
      return

    self._empty = False
    if 'indent' in self.attributes:
      output = json.dumps(element, indent=self.attributes['indent'])
    else:
      output = json.dumps(element, sort_keys=True)
    if delimit:
      self._out.write(self._delimiter + output)
      self._delimiter = ','
    else:
      if self._delimiter != self._BEGIN_DELIMITER:
        self._out.write(self._END_DELIMITER)
        self._delimiter = self._BEGIN_DELIMITER
      self._out.write(output)

  def Finish(self):
    if self._empty:
      if 'empty' in self.attributes:
        self._out.write(self._prefix + self.attributes['empty'])
    elif self._delimiter != self._BEGIN_DELIMITER:
      self._out.write(self._END_DELIMITER)
      self._delimiter = self._BEGIN_DELIMITER
    self._out.write('\n')
    super(GsutilJsonPrinter, self).Finish()



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing Compute Engine network edge security services."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import filter_rewrite
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties
from googlecloudsdk.core.resource import resource_projection_spec


@base.UniverseCompatible
class List(base.ListCommand):
  """List Compute Engine network edge security services.

  *{command}* list all network edge security services across all regions.

  ## EXAMPLES

  To list all network edge security services, run:

    $ {command}
  """

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""
        table(
          name,
          region.basename(),
          security_policy.basename()
        )
    """)

  def _GetListPage(self, network_edge_security_services, request):
    response = network_edge_security_services.AggregatedList(request)
    resource_lists = []
    for attachment_in_scope in response.items.additionalProperties:
      resource_lists += (attachment_in_scope.value.networkEdgeSecurityServices)
    return resource_lists, response.nextPageToken

  def Run(self, args):
    client = base_classes.ComputeApiHolder(
        self.ReleaseTrack()).client.apitools_client

    network_edge_security_services = client.networkEdgeSecurityServices

    messages = client.MESSAGES_MODULE
    project = properties.VALUES.core.project.GetOrFail()

    display_info = args.GetDisplayInfo()
    defaults = resource_projection_spec.ProjectionSpec(
        symbols=display_info.transforms, aliases=display_info.aliases)
    args.filter, filter_expr = filter_rewrite.Rewriter().Rewrite(
        args.filter, defaults=defaults)

    request = messages.ComputeNetworkEdgeSecurityServicesAggregatedListRequest(
        project=project, filter=filter_expr)
    if hasattr(request, 'returnPartialSuccess'):
      request.returnPartialSuccess = True

    # TODO(b/34871930): Write and use helper for handling listing.
    resource_lists, next_page_token = (
        self._GetListPage(network_edge_security_services, request))
    while next_page_token:
      request.pageToken = next_page_token
      resource_list_page, next_page_token = (
          self._GetListPage(network_edge_security_services, request))
      resource_lists += resource_list_page

    return resource_lists


List.detailed_help = base_classes.GetRegionalListerHelp(
    'network edge security services')



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud access-context-manager perimeters dry-run list` command."""


from googlecloudsdk.api_lib.accesscontextmanager import zones as zones_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.accesscontextmanager import policies
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class ListPerimeterDryRun(base.ListCommand):
  """Lists the effective dry-run configuration across all Service Perimeters."""
  _API_VERSION = 'v1'

  @staticmethod
  def Args(parser):
    base.URI_FLAG.RemoveFromParser(parser)
    parser.add_argument(
        '--policy',
        metavar='policy',
        default=None,
        help="""Policy resource - The access policy you want to list the
                effective dry-run configuration for. This represents a Cloud
                resource.""")
    parser.display_info.AddFormat('yaml(name.basename(), title, spec)')

  def Run(self, args):
    client = zones_api.Client(version=self._API_VERSION)
    policy_id = policies.GetDefaultPolicy()
    if args.IsSpecified('policy'):
      policy_id = args.policy

    policy_ref = resources.REGISTRY.Parse(
        policy_id, collection='accesscontextmanager.accessPolicies')

    perimeters_to_display = [p for p in client.List(policy_ref)]
    for p in perimeters_to_display:
      # When a Service Perimeter has use_explicit_dry_run_spec set to false, the
      # dry-run spec is implicitly the same as the status. In order to clearly
      # show the user what exactly is being used as the dry-run spec, we set
      # status to None (this list command is only for the dry-run config) and
      # copy over the status to the spec when the spec is absent. We also append
      # an asterisk to the name to signify that these perimeters are not
      # actually identical to what the API returns.
      if not p.useExplicitDryRunSpec:
        p.spec = p.status
        p.name += '*'
      p.status = None
    print('Note: Perimeters marked with \'*\' do not have an explicit `spec`. '
          'Instead, their `status` also acts as the `spec`.')
    return perimeters_to_display


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListPerimeterDryRunAlpha(ListPerimeterDryRun):
  """Lists the effective dry-run configuration across all Service Perimeters."""
  _API_VERSION = 'v1alpha'


detailed_help = {
    'brief': ('List the effective dry-run configuration across all Service '
              'Perimeters.'),
    'DESCRIPTION':
        ('By default, only the Service Perimeter name, title, type and the '
         'dry-run mode configuration (as `spec`) is displayed.\n\nNote: For '
         'Service Perimeters without an explicit dry-run mode configuration, '
         'the enforcement mode configuration is used as the dry-run mode '
         'configuration, resulting in no audit logs being generated.'),
    'EXAMPLES':
        ('To list the dry-run mode configuration across all Service '
         'Perimeter:\n\n  $ {command}\n\nOutput:\n\n  name: perimeter_1*\n  '
         'spec:\n    resources:\n    - projects/123\n    - projects/456\n    '
         'restrictedServices:\n    - storage.googleapis.com\n  title: Perimeter'
         ' 1\n  ---\n  name: perimeter_2\n  spec:\n    resources:\n    - '
         'projects/789\n    restrictedServices:\n    - '
         'bigquery.googleapis.com\n    vpcAccessibleServices:\n      '
         'allowedServices:\n      - bigquery.googleapis.com\n      '
         'enableRestriction: true\n  title: Perimeter 2')
}

ListPerimeterDryRunAlpha.detailed_help = detailed_help
ListPerimeterDryRun.detailed_help = detailed_help



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource args for batch-operations jobs commands."""


from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps as deps_lib
from googlecloudsdk.command_lib.util.concepts import concept_parsers

_SBO_CLH_LOCATION_GLOBAL = 'global'


def location_attribute_config():
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text=(
          'Batch-operations supported Google Cloud location for the {resource}.'
      ),
      fallthroughs=[
          # All batch operations jobs are sent to the global-clh.
          deps_lib.ValueFallthrough(_SBO_CLH_LOCATION_GLOBAL)
      ],
  )


def batch_job_attribute_config():
  return concepts.ResourceParameterAttributeConfig(
      name='batch-job', help_text='Batch Job ID for the {resource}.'
  )


def bucket_operation_attribute_config():
  return concepts.ResourceParameterAttributeConfig(
      name='bucket-operation',
      help_text='Bucket Operation ID for the {resource}.',
  )


def job_attribute_config():
  return concepts.ResourceParameterAttributeConfig(
      name='job', help_text='Batch Job ID for the {resource}.'
  )


def get_bucket_operation_resource_spec():
  """Returns a resource spec for a bucket operation."""
  return concepts.ResourceSpec(
      'storagebatchoperations.projects.locations.jobs.bucketOperations',
      resource_name='bucket-operation',
      bucketOperationsId=bucket_operation_attribute_config(),
      jobsId=job_attribute_config(),
      locationsId=location_attribute_config(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def add_bucket_operation_resource_arg(parser, verb):
  """Adds a resource argument for a bucket operation to describe."""
  concept_parsers.ConceptParser.ForResource(
      'bucket_operation',
      get_bucket_operation_resource_spec(),
      'The bucket operation to {}.'.format(verb),
      required=True,
      flag_name_overrides={'location': ''},
  ).AddToParser(parser)


def get_batch_job_resource_spec():
  return concepts.ResourceSpec(
      'storagebatchoperations.projects.locations.jobs',
      resource_name='batch-job',
      jobsId=batch_job_attribute_config(),
      locationsId=location_attribute_config(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
  )


def add_batch_job_resource_arg(parser, verb):
  """Adds a resource argument for storage batch-operations job.

  Args:
    parser: The argparser parser to add the resource arg to.
    verb: str, the verb to describe the resource, such as 'to create'
  """
  concept_parsers.ConceptParser.ForResource(
      'batch_job',
      get_batch_job_resource_spec(),
      'The batch job {}.'.format(verb),
      required=True,
      # Hide the location flag since all batch operations jobs are sent to the
      # global-clh and we don't want to expose this to the user.
      flag_name_overrides={'location': ''},
  ).AddToParser(parser)


def add_batch_job_flag_resource_arg(parser, verb):
  """Adds a resource argument for storage batch-operations job.

  Args:
    parser: The argparser parser to add the resource arg to.
    verb: str, the verb to describe the resource, such as 'to create'
  """
  concept_parsers.ConceptParser.ForResource(
      '--job',
      get_batch_job_resource_spec(),
      'The batch job {}.'.format(verb),
      required=True,
      flag_name_overrides={'location': ''},
  ).AddToParser(parser)
